#include <ifl/iflFile.h>
#include <ifl/iflFormat.h>
#include "iflJFIF.h"
#if defined(_WIN32)
#include <windows.h>
#include <io.h>
#define oserror() errno
#else
#include <unistd.h>
#if defined( SOLARIS_251 )
#include <strings.h>
#else
#include <bstring.h>
#endif
#endif
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <setjmp.h>
#include <errno.h>
#include <setjmp.h>
#include <ctype.h>
extern "C" {
#include "jpeglib.h"
}


class iflJFIFFile: public iflFile {
public:
    virtual int getNumImgs() { return nimgs; }
    virtual int getCurrentImg() { return imgidx; }
    virtual iflStatus setCurrentImg(int imgIdx);
    virtual iflStatus getItem(int tag, va_list ap);
    virtual iflStatus setItem(int tag, va_list ap);

private:
    virtual iflStatus openFile();
    virtual iflStatus createFile();
    virtual iflStatus closeFile();

    iflStatus _getICCProfile(int &size, void*& profile);
    virtual iflStatus _freeICCProfile(void* profile);
    iflStatus _setICCProfile(int size, const void* profile);

    friend class iflJFIFFormat;
    iflJFIFFile() {}	// only called by iflJFIFFormat::newfileobj()
    ~iflJFIFFile() {}	// only called by iflFile::close()
    
    virtual iflStatus getPage(void* data, int,int,int,int,int,int,int,int);
    virtual iflStatus setPage(const void* data, int,int,int,int,int,int,int,int);

    int nimgs, imgidx;

    FILE* ofile;
    
    // encoding parameters

    int compQuality;
    int optHuffTabEnabled;
    int restartInterval;
    int intInRows;
    int smoothingFactor;
    char* qtablefile;
    int hSample[4];	    // horizontal sampling factors
    int vSample[4];	    // vertical sampling factors
    int grayScaleEncoding;  // forces conversion to grayscale while encoding
    
    // decoding parameters

    void* dinfo;
    int blocksmooth;
    int grayScaleDecoding; // forces conversion to grayscale while decoding
    int first;
    
    // ICC info
    int profileSize;
    void* profileData;
};


class iflJFIFFormat : public iflFormat {
public:
    iflJFIFFormat() : iflFormat("JFIF") {}
    int typeIsSupported(iflDataType type) 
	{ return type == iflUChar; }
    int orderIsSupported(iflOrder order)
	{ return order == iflInterleaved; }
    int orientationIsSupported(iflOrientation orientation) 
	{ return orientation == iflUpperLeftOrigin; }
    int colorModelIsSupported(iflColorModel cm) 
	{ return cm == iflLuminance || cm == iflRGB || cm == iflCMYK; }
    int compressionIsSupported(iflCompression compress)
	{ return compress == iflJPEG; }
    int sizeIsSupported(int /*width*/, int /*height*/, int z, int c,
			iflOrientation) 
	{ return z == 1 && (c == 1 || c == 3 || c == 4); }
    int pagingIsSupported() { return 0; }
    int pageSizeIsSupported(int iWidth, int iHeight, int, int ic,
                            int pWidth, int pHeight, int pz, int pc, 
			    iflOrientation) 
	{ return pWidth == iWidth && pHeight == iHeight && 
	  pz == 1 && pc == ic; }
    int randomAccessReadIsSupported() { return 0; }
    int randomAccessWriteIsSupported() { return 0; }
    virtual iflDataType getPreferredType() { return iflUChar; }
    virtual iflOrder getPreferredOrder() { return iflInterleaved; }
    iflOrientation getPreferredOrientation() { return iflUpperLeftOrigin; }
    virtual iflColorModel getPreferredColorModel(int nc) { 
	switch (nc) {
	    case 1: 
	    case 2:  return iflLuminance;
	    case 4:  return iflCMYK; 
	    default: return iflRGB;
	}
    }
    virtual iflCompression getPreferredCompression() 
	{ return iflJPEG; }
    void getPreferredPageSize(int iWidth, int iHeight, int, int ic,
			      int& pWidth, int& pHeight, int& pz, int& pc, 
			      iflOrientation)
	{ pWidth = iWidth; pHeight = iHeight; pz = 1; pc = ic; }
    iflFile* newfileobj() { return new iflJFIFFile; }
    virtual iflTagDescription* _getTagDescriptions(int& count);
};


iflTagDescription*  
iflJFIFFormat::_getTagDescriptions(int& count)
{
    static iflIntRangeSpec rangeSpec = { 0, 100 };

    static iflTagDescription desc[] = {
      { iflJFIFcompressionQuality, 
    	iflTagGet|iflTagSet|iflTagIntRange|iflTagOneParam|iflTagCreate,
    	"The quality factor used in encoding (higher is better)",
    	"Compression quality",
    	"75",
    	&rangeSpec },
      { iflJFIFenableOptHuffTab, 
    	iflTagGet|iflTagSet|iflTagBoolean|iflTagOneParam|iflTagCreate
    	|iflTagHide,
    	"Enables optimization of entropy encoding parameters",
    	"Optimize entropy encoding",
    	"false",
    	NULL },
      { iflJFIFsmoothingFactor,
    	iflTagGet|iflTagSet|iflTagIntRange|iflTagOneParam|iflTagCreate,
    	"Smoothing factor to reduce dithering noise",
    	"Smoothing factor",
    	"0",
    	&rangeSpec },
      { iflJFIFgrayScaleEncoding, 
    	iflTagGet|iflTagSet|iflTagBoolean|iflTagOneParam|iflTagCreate
    	|iflTagHide,
    	"Forces encoding the result as greyscale image",
    	"Encode as grayscale",
    	"false",
    	NULL },
      { iflJFIFgrayScaleDecoding, 
    	iflTagGet|iflTagSet|iflTagBoolean|iflTagOneParam|iflTagOpen,
    	"Forces file to be decoded as grayscale image",
    	"Decode as grayscale",
    	"false",
    	NULL },
      { iflJFIFblockSmoothing, 
    	iflTagGet|iflTagSet|iflTagBoolean|iflTagOneParam|iflTagOpen,
    	"Enables cross-block smoothing (useful with low compression quality)",
    	"Block smoothing",
    	"false",
    	NULL }
    };

    count = sizeof desc / sizeof (iflTagDescription);
    return desc;
}


// This static object declaration causes the file format to be automatically
// registered when the DSO containing this code is dlopen'ed.
//
#if defined(__sgi)
#pragma set woff 1174	// 7.2 compiler doesn't understand /* REFERENCED */
#endif
/* REFERENCED */
static iflJFIFFormat theJFIFformat;
#if defined(__sgi)
#pragma reset woff 1174
#endif


// The IJG uses C's setjmp/longjmp facility to return control.  This means 
// that any routine which calls their library must first execute a setjmp() \
// call to establish the return point.  We want the replacement error_exit 
// to do a longjmp().  But we need to make the setjmp buffer accessible to the
// error_exit routine.  To do this, we make a private extension of the
// standard JPEG error handler object.  
//
struct ErrorMgr : public jpeg_error_mgr {
    inline ErrorMgr();
    int signaled() 
	{ return setjmp(setjmpBuffer); }

    jmp_buf setjmpBuffer;  // for return to caller
};

static void
errorExit(j_common_ptr cinfo)
{
    ErrorMgr* err = (ErrorMgr*)cinfo->err;

    // format the warning message 
    char buffer[JMSG_LENGTH_MAX];
    err->format_message(cinfo, buffer);
    
    iflError(MM_ERROR, "%s", buffer);

    // return control to the setjmp point
    longjmp(err->setjmpBuffer, 1);
}


// Decide whether to emit a trace or warning message.
// msgLevel is one of:
//   -1: recoverable corrupt-data warning, may want to abort.
//    0: important advisory messages (always display to user).
//    1: first level of tracing detail.
//    2,3,...: successively more detailed tracing messages.
//
static void
emitMessage (j_common_ptr cinfo, int msgLevel)
{
    jpeg_error_mgr* err = cinfo->err;

    if (msgLevel > 0) {
	// no trace messages
	return;    
    }
    else if (msgLevel < 0) {
	// It's a warning message.  Since corrupt files may generate many 
	// warnings, the policy implemented here is to show only the first 
	//
	err->num_warnings++;
	if (err->num_warnings > 1) return;
    }

    // format the warning message 
    char buffer[JMSG_LENGTH_MAX];
    err->format_message(cinfo, buffer);
    
    iflError(MM_WARNING, "%s", buffer);
}


ErrorMgr::ErrorMgr()
{
    // set up the normal JPEG error routines, then override error_exit
    jpeg_std_error(this);
    error_exit = errorExit;
    emit_message = emitMessage;
}


// read next char, skipping over any comments (# to end of line);
// a comment/newline sequence is returned as a newline 
//
static int
qtGetc(FILE* file)
{
    int ch = getc(file);
    if (ch == '#') {
	do ch = getc(file);
	while (ch != '\n' && ch != EOF);
    }

    return ch;
}


// read an unsigned decimal integer from a quantization-table file;
// swallows one trailing character after the integer 
//
static int
qtReadInteger(FILE* file)
{

    // skip any leading whitespace, detect EOF
    int ch;
    do {
	ch = qtGetc(file);
	if (ch == EOF) return EOF;
    } while (isspace(ch));
  
    if (!isdigit(ch)) {
	iflError(MM_ERROR, "bogus data in quantization file");
	return 0;
    }

    int val = ch - '0';
    while (ch = qtGetc(file), isdigit(ch)) {
	val *= 10;
	val += ch - '0';
    }

    return val;
}


// Read a set of quantization tables from the specified file.
// The file is plain ASCII text: decimal numbers with whitespace 
// between.  Comments preceded by '#' may be included in the file.
// There may be one to NUM_QUANT_TBLS tables in the file, each of 64 
// values. The tables are implicitly numbered 0,1,etc.
//
static iflStatus
readQuantTables(j_compress_ptr cinfo, char* filename, int scale_factor)
{
    // ZIG[i] is the zigzag-order position of the i'th element of a DCT block
    // read in natural order (left to right, top to bottom).
    static const int ZIG[DCTSIZE2] = {
	 0,  1,  5,  6, 14, 15, 27, 28,
	 2,  4,  7, 13, 16, 26, 29, 42,
	 3,  8, 12, 17, 25, 30, 41, 43,
	 9, 11, 18, 24, 31, 40, 44, 53,
	10, 19, 23, 32, 39, 45, 52, 54,
	20, 22, 33, 38, 46, 51, 55, 60,
	21, 34, 37, 47, 50, 56, 59, 61,
	35, 36, 48, 49, 57, 58, 62, 63
    };

    FILE* fp = fopen(filename, "r");
    if (fp == NULL) 
        return iflStatusEncode(iflOPENFAILED, iflSubDomainUNIX, oserror());
    
    unsigned int table[DCTSIZE2];
    int tblno = 0;
    int val;

    while ((val = qtReadInteger(fp)) != EOF) { // read 1st element of table 
	if (tblno >= NUM_QUANT_TBLS) {
	    iflError(MM_ERROR, "too many tables in file %s", filename);
	    return iflStatusEncode(iflBADIMGFMT);
	}
	table[0] = (unsigned int)val;
	for (int i = 1; i < DCTSIZE2; i++) {
	    if ((val = qtReadInteger(fp)) == EOF) {
		iflError(MM_ERROR, "incomplete table in file %s", filename);
		return iflStatusEncode(iflBADIMGFMT);
	    }
	    table[ZIG[i]] = (unsigned int)val;
	}
	jpeg_add_quant_table(cinfo, tblno, table, scale_factor, TRUE);
	tblno++;
    }

    fclose(fp);
    
    return iflOKAY;
}


static int
readHeader(j_decompress_ptr cinfo, FILE *infile)
{
    // use our private extension JPEG error handler
    ErrorMgr jerr;
    cinfo->err = &jerr;
    if (jerr.signaled()) {
	jpeg_destroy_decompress(cinfo);
	return 0;
    }

    // now we can initialize the JPEG decompression object.
    jpeg_create_decompress(cinfo);

    // specify data source (eg, a file)

    jpeg_stdio_src(cinfo, infile);

    // read file parameters 
    jpeg_read_header(cinfo, TRUE);
    
    return 1;
}


iflStatus 
iflJFIFFile::createFile()
{
    // init dynamically allocate members in case we prematurely abort
    qtablefile = NULL;
    dinfo = NULL;
    ofile = NULL;
    profileData = NULL;
    
    // create the file
    int fd_opened_here = 0;
    if (fd < 0) {
	assert(filename != NULL);
#   	if defined(_WIN32)
	fd = ::open(filename, accessmode|O_CREAT|O_TRUNC|O_BINARY, 0666);
#   	else
	fd = ::open(filename, accessmode|O_CREAT|O_TRUNC, 0666);
#   	endif
	if (fd < 0) return iflStatusEncode(iflOPENFAILED, iflSubDomainUNIX, 
					   oserror());
	fd_opened_here = 1;
    } 
    else {
#   	if !defined(_WIN32)
	(void)::ftruncate(fd, (off_t)0);
#   	endif
    }
    if ((ofile = fdopen(fd, "w+")) == NULL) {
	if (fd_opened_here) {
	    (void)::close(fd);
	    fd = -1;
	}
	return iflStatusEncode(iflOPENFAILED, iflSubDomainUNIX, oserror());
    }

    // fill in info
    nimgs = 1;
    imgidx = 0;
    switch (size.c) {
	case 1:  cm = iflLuminance; break;
	case 3:  cm = iflRGB; break;
	case 4:  cm = iflCMYK; break;
	default: cm = iflColorModel(0);
		 break;
    }

    // encoding parametrs 
    
    compQuality = 75;
    grayScaleEncoding = FALSE;
    optHuffTabEnabled = FALSE;
    restartInterval = 0;
    intInRows = TRUE;
    smoothingFactor = 0;
    hSample[0] = vSample[0] = 2;
    hSample[1] = hSample[2] = hSample[3] =
    vSample[1] = vSample[2] = vSample[3] = 1;

    // decoding parameters 

    blocksmooth = FALSE;
    grayScaleDecoding = FALSE;
    
    return iflOKAY;
}


iflStatus 
iflJFIFFile::openFile()
{
    qtablefile = NULL;
    dinfo = NULL;
    ofile = NULL;
    profileData = NULL;

    // open the file
    int fd_opened_here = 0;
    if (fd < 0) {
	assert(filename != NULL);
#   	if defined(_WIN32)
	fd = ::open(filename, accessmode|O_BINARY);
#   	else
	fd = ::open(filename, accessmode);
#   	endif
	if (fd < 0)
	    return iflStatusEncode(iflOPENFAILED, iflSubDomainUNIX, oserror());
	fd_opened_here = 1;
    }
    else
	// already opened, make sure we are at beginning of file
	lseek(fd, 0, SEEK_SET);
    if ((ofile = fdopen(fd, "r")) == NULL) {
	if (fd_opened_here) {
	    (void)::close(fd);
	    fd = -1;
	}
	return iflStatusEncode(iflOPENFAILED, iflSubDomainUNIX, oserror());
    }

    // do common initializations
    jpeg_decompress_struct* cinfo = new jpeg_decompress_struct;
    dinfo = cinfo;

    // read the header
    if (!readHeader(cinfo, ofile)) {
	delete cinfo;
	if (fd_opened_here) {
	    (void)::close(fd);
	    fd = -1;
	}
	return iflStatusEncode(iflOPENFAILED, iflSubDomainUNIX, oserror());
    } else {
	// we read the header.  if there was an embedded ICC device profile,
	// it's in the cinfo.  We've got to read it out now. 
	if (cinfo->prof_info && cinfo->prof_info->length != 0) {
	    profileSize = (int)cinfo->prof_info->length;
	    profileData = new char[profileSize];
	    if (profileData) 
	    	memcpy(profileData, cinfo->prof_info->profile, profileSize);
	}
    }
    first = 1;

    nimgs = 4;
    setCurrentImg(3);	// Base

    // extract the image attributes from the file header
    size.x = cinfo->image_width;
    size.y = cinfo->image_height;
    size.z = 1;
    dtype = iflUChar;
    order = iflInterleaved;
    orientation = iflUpperLeftOrigin;
    compression = iflJPEG;

    if (cinfo->num_components == 1) cinfo->out_color_space = JCS_GRAYSCALE;
    switch (cinfo->out_color_space) {

    case JCS_GRAYSCALE:
	cm = iflLuminance;
	size.c = 1;
	break;
    case JCS_RGB:
	cm = iflRGB;
	size.c = 3;
	break;
    case JCS_CMYK:
	cm = iflCMYK;
	size.c = 4;
	break;
    case JCS_YCbCr:
    case JCS_YCCK:
    case JCS_UNKNOWN:
	cm = iflColorModel(0);
	size.c = 0;
	if (fd_opened_here) {
	    (void)::close(fd);
	    fd = -1;
	}
	return iflStatusEncode(iflUNSUPPORTEDBYLIBRARY);
    }

    pageSize = size;

    // decoding parameters
    blocksmooth = FALSE;
    grayScaleDecoding = FALSE;
    
    return iflOKAY;
}


iflStatus
iflJFIFFile::closeFile()
{
    if (ofile != NULL) fclose(ofile);
    if (fd >= 0) ::close(fd);

    if (qtablefile) free(qtablefile);
    if (dinfo) {
	jpeg_destroy_decompress((j_decompress_ptr)dinfo);
	delete (j_decompress_ptr)dinfo;
    }
    
    delete profileData;
    
    return iflOKAY;
}


iflStatus 
iflJFIFFile::getPage(void* data, int, int, int, int, int, int, int, int)
{
    jpeg_decompress_struct* cinfo = (jpeg_decompress_struct*)dinfo;

    // file is empty, just return
    if (cinfo == NULL)
	return iflOKAY;

    // use our private extension JPEG error handler
    ErrorMgr jerr;
    cinfo->err = &jerr;
    if (jerr.signaled()) {
	jpeg_destroy_decompress(cinfo);
	return iflStatusEncode(iflREADFAILED);
    }

    // set parameters for decompression
    if (grayScaleDecoding) cinfo->out_color_space = JCS_GRAYSCALE;
    if (blocksmooth) cinfo->do_fancy_upsampling = TRUE;
    if (!first) {
    	rewind(ofile);
    	jpeg_read_header(cinfo, TRUE);
    	setCurrentImg(imgidx);
    }
    jpeg_start_decompress(cinfo);

    // read the lines in
    int rowStride = (cinfo->output_width*cinfo->num_components);
    unsigned char* cdata = (unsigned char*)data;
    while (cinfo->output_scanline < cinfo->output_height) {
	int numread = jpeg_read_scanlines(cinfo, &cdata, 
					  cinfo->rec_outbuf_height);
	cdata += numread*rowStride;
    }
    
    jpeg_finish_decompress(cinfo);
    first = 0;

    return iflOKAY;
}


iflStatus 
iflJFIFFile::setPage(const void* data, int, int, int, int, int, int, int, int)
{
    iflStatus sts = iflOKAY;

    // allocate and initialize JPEG compression object
    //
    struct jpeg_compress_struct cinfo;

    // use our private extension JPEG error handler
    ErrorMgr jerr;
    cinfo.err = &jerr;
    if (jerr.signaled()) {
	jpeg_destroy_compress(&cinfo);
	return iflStatusEncode(iflWRITEFAILED);;
    }

    // Now we can initialize the JPEG compression object
    jpeg_create_compress(&cinfo);
    jpeg_stdio_dest(&cinfo, ofile);

    // set parameters for compression:  first we supply a description of 
    // the input image.
    //
    cinfo.image_width = size.x; 
    cinfo.image_height = size.y;
    cinfo.input_components = size.c;
    switch (size.c) {
	case 1: cinfo.in_color_space = JCS_GRAYSCALE; break;
	case 3: cinfo.in_color_space = JCS_RGB; break;
	case 4: cinfo.in_color_space = JCS_CMYK; break;
	default: cinfo.in_color_space = JCS_RGB; break;
    }

    // now use the library's routine to set default compression parameters.
    //
    jpeg_set_defaults(&cinfo);

    // now set non-default parameters 
    //
    jpeg_set_quality(&cinfo, compQuality, TRUE);
    int qScaleFactor = jpeg_quality_scaling(compQuality);
    if (optHuffTabEnabled) cinfo.optimize_coding = TRUE;
    if (intInRows) 
	cinfo.restart_in_rows = restartInterval;
    else {
	cinfo.restart_interval = (unsigned int)restartInterval;
	cinfo.restart_in_rows = 0; 
    }
    cinfo.comp_info[0].h_samp_factor = hSample[0];
    cinfo.comp_info[0].v_samp_factor = vSample[0];
    cinfo.comp_info[1].h_samp_factor = hSample[1];
    cinfo.comp_info[1].v_samp_factor = vSample[1];
    cinfo.comp_info[2].h_samp_factor = hSample[2];
    cinfo.comp_info[2].v_samp_factor = vSample[2];
    cinfo.comp_info[3].h_samp_factor = hSample[3];
    cinfo.comp_info[3].v_samp_factor = vSample[3];
    cinfo.smoothing_factor = smoothingFactor;
    if (qtablefile != NULL) 
	sts = readQuantTables(&cinfo, qtablefile, qScaleFactor);

    // start compressor; TRUE ensures that we will write a complete 
    // interchange-JPEG file.
    //
    if (sts == iflOKAY) {
	jpeg_start_compress(&cinfo, TRUE);

#ifdef ICC_SUPPORT
        if (profileSize > 0 && profileData != NULL)
	    write_icc_profile(&cinfo, (JOCTET*)profileData, profileSize);
#endif
    
	// write the data
	int rowStride = cinfo.image_width*size.c;
	unsigned char* cdata = (unsigned char*)data;
	while (cinfo.next_scanline < cinfo.image_height) {
	    int numread = jpeg_write_scanlines(&cinfo, &cdata, 1);
	    cdata += numread*rowStride;
	}
    
	// finish compression
	//
	jpeg_finish_compress(&cinfo);
    }

    jpeg_destroy_compress(&cinfo);

    return sts;
}


iflStatus
iflJFIFFile::setCurrentImg(int imgIdx)
{
    if (imgIdx < 0 || imgIdx > 3) 
	return iflStatusEncode(iflUNSUPPORTEDBYLIBRARY);

    j_decompress_ptr dmcp = (j_decompress_ptr)dinfo;
    int scale;
    switch (imgidx = imgIdx) {
    case 0:
	dmcp->scale_num = 1;   
	dmcp->scale_denom = 8;
	scale = 8;
	break;   
    case 1:
	dmcp->scale_num = 1;   
	dmcp->scale_denom = 4;
	scale = 4;
	break;   
    case 2:
	dmcp->scale_num = 1;   
	dmcp->scale_denom = 2;
	scale = 2;
	break;   
    case 3:
	dmcp->scale_num = 1;   
	dmcp->scale_denom = 1;
	scale = 1;
	break;
    }

    dmcp->output_width = (dmcp->image_width + scale - 1)/scale;
    dmcp->output_height = (dmcp->image_height + scale - 1)/scale;
    pageSize.x = size.x = dmcp->output_width;
    pageSize.y = size.y = dmcp->output_height;
    
    return iflOKAY;
}


iflStatus
iflJFIFFile::getItem(int tag, va_list ap)
{
#define ReturnTag(type, name) \
    if (1) { \
	type* val = va_arg(ap, type*); \
	if (val == NULL) return iflStatusEncode(iflBADPARAMS); \
	*val = name; \
	return iflOKAY; \
    } else

    switch (tag) {

    case iflJFIFcompressionQuality:
	ReturnTag(int, compQuality);

    case iflJFIFenableOptHuffTab:
	ReturnTag(int, optHuffTabEnabled);

    case iflJFIFrestartInterval:
	ReturnTag(int, restartInterval);

    case iflJFIFsmoothingFactor:
	ReturnTag(int, smoothingFactor);

    case iflJFIFqTable:
	ReturnTag(char*, qtablefile);

    case iflJFIFsamplingFactors:
      {
	int *horiz = va_arg(ap, int*), *vert = va_arg(ap, int*);
	int ch = va_arg(ap, int);
	if (ch < 0 || ch > 3)
	    return iflStatusEncode(iflBADPARAMS);
	*horiz = hSample[ch];
	*vert = vSample[ch];
	
	return iflOKAY; 
      }

    case iflJFIFgrayScaleEncoding:
	ReturnTag(int, grayScaleEncoding);

    case iflJFIFgrayScaleDecoding:
	ReturnTag(int, grayScaleDecoding);

    case iflJFIFblockSmoothing:
	ReturnTag(int, blocksmooth);

    default:
	return iflStatusEncode(iflBADPARAMS);
    }
}


iflStatus
iflJFIFFile::setItem(int tag, va_list ap)
{
    switch (tag) {

    case iflJFIFcompressionQuality:
      {
	int val = va_arg(ap, int);
	if (val < 0 || val > 100) return iflStatusEncode(iflBADPARAMS);
	compQuality = val;
	return iflOKAY;
      }

    case iflJFIFenableOptHuffTab:
	optHuffTabEnabled = va_arg(ap, int);
	return iflOKAY;

    case iflJFIFrestartInterval:
	restartInterval = va_arg(ap, int);
	intInRows = va_arg(ap, int);
	return iflOKAY;

    case iflJFIFsmoothingFactor:
      {
	int val = va_arg(ap, int);
	if (val < 0 || val > 100) return iflStatusEncode(iflBADPARAMS);
	smoothingFactor = val;
	return iflOKAY;
      }

    case iflJFIFqTable:
      {
	if (qtablefile) { free(qtablefile); qtablefile = NULL; }
	char* filename = va_arg(ap, char*);
	if (filename) qtablefile = strdup(filename);
	return iflOKAY;
      }

    case iflJFIFsamplingFactors:
      {
	int horiz = va_arg(ap, int), vert = va_arg(ap, int);
	int ch = va_arg(ap, int);
	if (ch < 0 || ch > 3)
	    return iflStatusEncode(iflBADPARAMS);
	if (horiz <= 0 || horiz > 4 || vert <= 0 || vert > 4)
	    return iflStatusEncode(iflBADPARAMS);
	
	hSample[ch] = horiz;
	vSample[ch] = vert;
	
	return iflOKAY; 
      }

    case iflJFIFgrayScaleEncoding:
	grayScaleEncoding = va_arg(ap, int);
	return iflOKAY;

    case iflJFIFgrayScaleDecoding: 
      {
	int oldDecoding = grayScaleDecoding;
	grayScaleDecoding = va_arg(ap, int);
	if (grayScaleDecoding != oldDecoding) {
	    if (grayScaleDecoding)
		{ size.c = 1; cm = iflLuminance; }
	    else
		{ size.c = 3; cm = iflRGB; }
	    markAttributesChanged();
	}
	return iflOKAY;
      }

    case iflJFIFblockSmoothing:
      {
	int oldSmooth = blocksmooth;
	blocksmooth = va_arg(ap, int);
	if (blocksmooth != oldSmooth) markAttributesChanged();
	return iflOKAY;
      }

    default:
	return iflStatusEncode(iflBADPARAMS);
    }
}


iflStatus
iflJFIFFile::_getICCProfile(int& size, void*& profile)
{
    if (profileSize != 0 && profileData != NULL) {
	size = profileSize;
	profile = profileData;
	return iflOKAY;
    }

    // initialize these anyway, in case they forget to error check
    size = 0;
    profile = NULL;
    return iflStatusEncode(iflBADPARAMS);
}


iflStatus 
iflJFIFFile::_freeICCProfile(void*)
{
    // don't do anything, we manage the memory internally
    return iflOKAY;
}


iflStatus
iflJFIFFile::_setICCProfile(int size, const void* profile)
{
    delete profileData;
    profileData = new char[size];
    if (profileData == NULL) return iflStatusEncode(iflMALLOCFAILED);
	
    profileSize = size;
    memcpy(profileData, profile, size);

    return (iflOKAY);
}
