#ifndef OPTEONGLUE_H
#define OPTEONGLUE_H

// This file is meant to be somewhat API compatible with vid.h which I use for SGI machines
#include "jd_time.h"
#include "depict.h"
#include "JDImage.h"
#include <queue>

// Dont invoke the class directly, use vidInit()
class VidInfo
{
public:
	int count;
	unsigned long basetime;
	DEPICT_HANDLE m_hCamera;
	void * m_pvImage[10];
	void *pvRGB32Image;
	DEPICT_ACQ_STATUS sStatus[10];
	LARGE_INTEGER sTriggerTimestamp[10];
	unsigned long prevendtime;
	int idlecount;
	ULONG m_ulImageWidth;
	ULONG m_ulImageHeight;
	ULONG ulCameraID;
	DEPICT_TRIGGER eTriggerMode;
	VidInfo() {idlecount=0; count=0; basetime=Get_Milliseconds(); eTriggerMode=DepictTriggerImmediate;}
	int thisCameraNumber;
	std::queue<int> bufferQueue;
};

typedef void RawImage;

VidInfo *vidInit(int camNum=0);
void     vidQuit(VidInfo *info);
char    *vidIn(VidInfo *info);
void vidIn(VidInfo *info, RawImage *im);
void vidIn(VidInfo *info, JDImage &im);
void vidIn_BGRA(VidInfo *info, JDImage &im);

//char    *vidInLatest(VidInfo *info, int *sequence=NULL);
void     vidGetXYSize(VidInfo *info, long *x, long *y);
//void     vidSetXYSize(VidInfo *info, int x, int y);
//void     vidDeleteBuffer(VidInfo *info);
//void     vidCreateBuffer(VidInfo *info);
//void vidSetZoom(VidInfo *info, int num, int den);
//void vidSetCapType(VidInfo *info, int captype);
//void vidSetRate(VidInfo *info, int rate);
//void vidSetPacking(VidInfo *info, int packing);

/* Note that this expires all these buffers while obtaining the count */
//int  vidBuffersPending(VidInfo *info);

// This pair must come as a matched set. However you must make all master/slave
// calls to vidStartCapture before making all calls to vidFinishCapture
// There is no forward buffering to speed things up going on here
char * vidFinishCapture(VidInfo *info);
void vidStartCapture(VidInfo *info);

ULONG vidGetNumCameras();

void vidSetShutterTimeMicroSec(VidInfo *vid, long ms);
void vidSetPipelineMode(VidInfo *info);

// Camera syncing
void vidSetSlaveToMaster(VidInfo *slave, VidInfo *master);

// Handle space for allocating images
RawImage *AllocateRawImageSpace(VidInfo *vid);
void DeleteRawImageSpace(RawImage *im);
void PreAllocateRawImageSpace(VidInfo *vid, int num);

// Format conversion
void convertRawImageToJDImage(VidInfo *vid, RawImage *im, JDImage &jdim);
void convertRawImageToJDImageBGRA(VidInfo *vid, RawImage *im, JDImage &jdim);

#endif 

