
#include "FImage.h"
#include <stdlib.h>
#include "memory.h"
#include <string.h>
#include "jed.h"
#include "iflinterface.h"

using namespace std;

void FImage::Init(void)
{
  fPixels=NULL;
  fXSize=0;
  fYSize=0;
}

FImage::FImage(void)
{
Init();
}

FImage::FImage(short x, short y)
{
Init();
SetSize(x,y);
}

FImage::FImage(ImageInfo info)
{
  Init();
  *this = info;
}

FImage::FImage(FImage &fim)
{
  Init();
  SetSize(fim.fXSize,fim.fYSize);
  memcpy(fPixels,fim.fPixels,fXSize*fYSize*sizeof(float));
  
}

FImage &FImage::operator=(ImageInfo info)
{
  SetSize(info.xsize,info.ysize);
  for (int x=0;x<fXSize;x++)
    for(int y=0; y<fYSize;y++)
      {
	Pix(x,y)=info.r(x,y);
      }
  
  return *this;
}


void FImage::SetSize(short x, short y)
{
  if (fPixels!=NULL)
    {
      DisposPtr((Ptr)fPixels);
    }
	
  fPixels=(float *)NewPtr(sizeof(float) * x * y);

  if (fPixels!=NULL) 	
    {
      fXSize=x;
      fYSize=y;
    }

}

void FImage::getJDImage(JDImage &im)
{
  im.resize(fXSize,fYSize);
  for (int x=0;x<fXSize;x++)
    for(int y=0; y<fYSize;y++)
      {
	im.r(x,y)=Pix(x,y);
	im.g(x,y)=Pix(x,y);
	im.b(x,y)=Pix(x,y);
      }
}

void FImage::blurX(Kernel &kernel)
{

  // Check that the kernel has an odd size
  if (kernel.size()%2 != 1) {
    COUT << "KERNEL SIZE should be odd";
    return;
  }

  // Create a temporary holder
  FImage tim(*this);
  
  // What is the kernel half width
  int half=(kernel.size())/2;
  
  // Loop safely over things near boundaries
  // (everything for now, but could improve the speed of this)

  // loop over pixels
  for (int y=0; y <fYSize; y++)
    for (int x=0; x <fXSize; x++)
      {
	// Start accumulating
	double tot=0;
	
	// loop over kernel
	for (int k= -half ;k<=half; k++) {
	  
	  tot += (kernel[k+half] * tim.ClampedPix(x+k,y));
	  // COUT << x << "   y:" << y << "   tot:" << tot  << "  size:"
	  //     << kernel.size() << "  half:" << half << "   k+half:"
	  //     << k+half << "   x+k:" << x+k << endl;

	}

	// Set new value
	
	Pix(x,y)=tot;
      }

  
}

void FImage::blurY(Kernel &kernel)
{

  // Check that the kernel has an odd size
  if (kernel.size()%2 != 1) {
    COUT << "KERNEL SIZE should be odd";
    return;
  }

  // Create a temporary holder
  FImage tim(*this);
  
  // What is the kernel half width
  int half=(kernel.size())/2;
  
  // Loop safely over things near boundaries
  // (everything for now, but could improve the speed of this)

  // loop over pixels
  for (int y=0; y <fYSize; y++)
    for (int x=0; x <fXSize; x++)
      {
	// Start accumulating
	double tot=0;
	
	// loop over kernel
	for (int k= -half ;k<=half; k++) {
	  
	  tot += (kernel[k+half] * tim.ClampedPix(x,y+k));
	  // COUT << x << "   y:" << y << "   tot:" << tot  << "  size:"
	  //     << kernel.size() << "  half:" << half << "   k+half:"
	  //     << k+half << "   x+k:" << x+k << endl;

	}

	// Set new value
	
	Pix(x,y)=tot;
      }

  
}

void FImage::blur(Kernel &kernel)
{
  blurX(kernel);
  blurY(kernel);
  
}
