/**********************************************************************
* DEPICT.H - public include file for Rendition Depict Imaging Subsystem
*
**********************************************************************/

/* Guard ifndef */
#ifndef __DEPICT_H__
#define __DEPICT_H__

/* Includes */
#include <windows.h>

/* DLL import declarations */
#if defined( __BORLAND_C__ ) /* Warning: borland not tested */
  #if ! defined( _DEPICT_API )
    #define _DEPICT_API _import PASCAL
  #endif
#else /* microsoft and the like are currently supported */
  #if ! defined( _DEPICT_API )
    #define _DEPICT_API __declspec(dllimport)
  #endif
#endif


/*************************************
* Camera and Platform ID Definitions *
*************************************/

/* Camera Platform Types */
typedef enum _DEPICT_PLATFORM_TYPE
{
   DepictPlatformTypeUnknown        = -1,
   DepictPlatformTypeClassic        = 0,
   DepictPlatformTypeSSeries        = 1, 
   DepictPlatformTypeUSB            = 2,
   DepictPlatformTypeEthernet       = 3,
   DepictPlatformTypeMAXIMUM        = 4
} DEPICT_PLATFORM_TYPE;

/* Macros for creating Platform IDs */
#define DEPICT_PLATFORM_ID_INVALID               0xFFFFFFFF

#define DEPICT_MAKE_PLATFORM_ID(PlatformType, PlatformNumber)    ((UCHAR(PlatformType) << 8) | UCHAR(PlatformNumber))
#define DEPICT_PLATFORM_ID_UNKNOWN                       DEPICT_MAKE_PLATFORM_ID(DepictPlatformTypeUnknown, 0)
#define DEPICT_PLATFORM_ID_CLASSIC(PlatformNumber)       DEPICT_MAKE_PLATFORM_ID(DepictPlatformTypeClassic, PlatformNumber)
#define DEPICT_PLATFORM_ID_SSERIES(PlatformNumber)       DEPICT_MAKE_PLATFORM_ID(DepictPlatformTypeSSeries, PlatformNumber)
#define DEPICT_PLATFORM_ID_USB                           DEPICT_MAKE_PLATFORM_ID(DepictPlatformTypeUSB, 0)
#define DEPICT_PLATFORM_ID_ETHERNET(PlatformNumber)      DEPICT_MAKE_PLATFORM_ID(DepictPlatformTypeEthernet, PlatformNumber)

/* Macros for extracting platform Type and Number info from Platform IDs */
#define DEPICT_GET_PLATFORM_TYPE(id)    DEPICT_PLATFORM_TYPE(char((id) >> 8))
#define DEPICT_GET_PLATFORM_NUMBER(id)  (char((id) & 0xFF))

/* Definition of invalid serial number identifier */
#define DEPICT_SERIAL_NUMBER_INVALID 0xFFFFFFFF

/* Legacy board number definitions from earlier Depict versions.
   These would now be replaced by: CLASSIC_PLATFORM_ID(n),
   where "n" is the 0-indexed board number. */
#define DEPICT_BOARD_1  0
#define DEPICT_BOARD_2  1
#define DEPICT_BOARD_3  2
#define DEPICT_BOARD_4  3
#define DEPICT_BOARD_5  4
#define DEPICT_BOARD_6  5
#define DEPICT_BOARD_7  6
#define DEPICT_BOARD_8  7
#define DEPICT_BOARD_9  8
#define DEPICT_BOARD_10 9

/* Legacy camera number definitions from earlier Depict versions.
   These are replaced by the 0-indexed camera number */
#define DEPICT_CAMERA_NONE       -1
#define DEPICT_CAMERA_1          0
#define DEPICT_CAMERA_2          1
#define DEPICT_CAMERA_3          2
#define DEPICT_CAMERA_4          3 


/************************* 
* Depict parameter types *
**************************/

/* Depict Device Handle */
typedef ULONG DEPICT_HANDLE;
typedef ULONG DEPICT_CAMERA_HANDLE;

#define DEPICT_INVALID_HANDLE 0

/* Acquisition Status */
typedef struct 
{
   HANDLE hEvent;                     /* acq complete notification */
   HANDLE hTriggerEvent;              /* acq trigger notification */ 
   LARGE_INTEGER * pTriggerTimestamp; /* acq trigger timestamp */
   ULONG anReserved[24];              /* Reserved */
} DEPICT_ACQ_STATUS;

/* I/O Status */
typedef struct 
{
   HANDLE hEvent;                     /* I/O complete notification */
   ULONG anReserved[8];               /* Reserved */
} DEPICT_IO_STATUS;

/* Acquisition Trigger Types */
typedef enum 
{
   DepictTriggerImmediate = 1,   /* indicates immediate trigger */
   DepictTriggerTimebase = 2,    /* indicates timebase trigger */
   DepictTriggerExternalA = 3,   /* indicates external trigger A */
   DepictTriggerExternalB = 4,   /* indicates external trigger B */
   DepictTriggerMaster = 5,      /* indicates external master trigger */
   DepictTriggerROI = 6,         /* indicates trigger on ROI */
   DepictTriggerSlave = 7,       /* indicates trigger on master camera */
   DepictTriggerLSExtLine = 8,   /* indicates external trigger for line scan line */
   DepictTriggerLSExtFrame = 9,   /* indicates external trigger for line scan frame */
   DepictTriggerLSExtLineFrame = 10,   /* indicates external trigger for line scan line and frame */
} DEPICT_TRIGGER, *PDEPICT_TRIGGER;

/* Strobe Types */
typedef enum 
{
   DepictStrobeOnPrimaryRoi = 0, /* strobe on primary ROI only */
   DepictNoStrobe = 1,           /* no strobe */
   DepictStrobeOnBoth = 2,       /* strobe on primary and AOI ROI */
   DepictStrobeOnAoi = 3,        /* strobe on AOI only */
} DEPICT_STROBE, *PDEPICT_STROBE;

/* AOI Trigger Level Threshold Types */
typedef enum 
{
   DepictThresholdGreater = 1,  /* indicates greater than Threshold1 */
   DepictThresholdLess = 2,     /* indicates less than Threshold2    */
   DepictThresholdBetween = 3   /* indicates between Threshold1 and Threshold2 */
} DEPICT_THRESHOLD, *PDEPICT_THRESHOLD;

/* AOI Trigger Std. Dev. Threshold Types */
typedef enum 
{
   DepictDeviationBelow = 0,         /* below standard deviation */
   DepictDeviationAbove = 1          /* above standard deviation */
} DEPICT_DEVIATION_REQUIREMENT, *PDEPICT_DEVIATION_REQUIREMENT;

/* Image Format Types */
typedef enum 
{
   DepictFormatUnknown = -1,         /* Unknown Format */
   DepictFormatNone = 0,             /* Format not applicable */
   DepictFormatGrayscale = 1,        /* 256 gray-scale */
   DepictFormatRGB16 = 2,            /* 16-bit RGB */
   DepictFormatRGB24 = 3,            /* 24-bit RGB */
   DepictFormatRGB32 = 4,            /* 32-bit RGB */
   DepictFormatYUV1 = 5,             /* YUV format one */
   DepictFormatYUV2 = 6,             /* YUV format two */
   DepictFormat10BitRGB32 = 7,       /* 10 bits-per-channel RGB in 32-bits-per-pixel format */
   DepictFormat8BitBayerPattern = 8  /* 8-bit Bayer-pattern colour */
} DEPICT_FORMAT, *PDEPICT_FORMAT;

/* Camera sensor type */
typedef enum
{
   DepictCameraSensorB1A = 0,
   DepictCameraSensorC1A = 1,
   DepictCameraSensorC3A = 2,
   DepictCameraSensorB2D = 3,
   DepictCameraSensorB1E = 4,
   DepictCameraSensorC1E = 5,
   DepictCameraSensorB2F = 6,
   DepictCameraSensorC2F = 7,
   DepictCameraSensorC3F = 8,
   DepictCameraSensorB4G = 9,
   DepictCameraSensorB2H = 10
} DEPICT_CAMERA_SENSOR;

/* Camera imager color type */
typedef enum
{
   DepictCameraSensorColorGrayscale = 0,
   DepictCameraSensorColorBayer = 1,
   DepictCameraSensorColor3Chip = 2
} DEPICT_CAMERA_SENSOR_COLOR;

/* Camera channels */
#define DEPICT_CAMERA_CHANNEL_ALL 0
#define DEPICT_CAMERA_CHANNEL_STANDARD 0
#define DEPICT_CAMERA_CHANNEL_RED 1
#define DEPICT_CAMERA_CHANNEL_GREEN 2
#define DEPICT_CAMERA_CHANNEL_BLUE 3
#define DEPICT_CAMERA_CHANNEL_TAP_1 4
#define DEPICT_CAMERA_CHANNEL_TAP_2 8
#define DEPICT_CAMERA_CHANNEL_TAP_3 12
#define DEPICT_CAMERA_CHANNEL_TAP_4 16

/* Camera bundles */
#define DEPICT_CAMERA_BUNDLE_STANDARD  0
#define DEPICT_CAMERA_BUNDLE_TRIGGER   1
#define DEPICT_CAMERA_BUNDLE_FIDELITY  2
#define DEPICT_CAMERA_BUNDLE_AGILITY   4

/* Camera special function flags */
#define DEPICT_CAMERA_FUNCTION_NONE          0
#define DEPICT_CAMERA_FUNCTION_MPEG_II       1
#define DEPICT_CAMERA_FUNCTION_COLOR_PROC    2


/*********************
* Depict Error Codes *
*********************/

typedef ULONG DEPICT_RESULT;

#define DEPICT_SUCCESS                   ERROR_SUCCESS
#define DEPICT_ERROR_SUCCESS             ERROR_SUCCESS

#define DEPICT_MAKE_WARNING(code)        (code | 0x80000000 | 0x20000000)
#define DEPICT_MAKE_ERROR(code)          (code | 0xC0000000 | 0x20000000)

#define DEPICT_WARNING_ROI_WAS_ADJUSTED  DEPICT_MAKE_WARNING(1)
#define DEPICT_WARNING_GAIN_AT_MIN       DEPICT_MAKE_WARNING(2)
#define DEPICT_WARNING_GAIN_AT_MAX       DEPICT_MAKE_WARNING(3)
#define DEPICT_WARNING_OFFSET_AT_MIN     DEPICT_MAKE_WARNING(4)
#define DEPICT_WARNING_OFFSET_AT_MAX     DEPICT_MAKE_WARNING(5)

#define DEPICT_ERROR_UNKNOWN             DEPICT_MAKE_ERROR(0)
#define DEPICT_ERROR_ILLEGAL_VALUE       DEPICT_MAKE_ERROR(1)
#define DEPICT_ERROR_NOT_SUPPORTED       DEPICT_MAKE_ERROR(2)
#define DEPICT_ERROR_TIMEOUT             DEPICT_MAKE_ERROR(3)
#define DEPICT_ERROR_DEVICE_NOT_OPEN     DEPICT_MAKE_ERROR(10)
#define DEPICT_ERROR_NO_CAMERA_FOUND     DEPICT_MAKE_ERROR(10)
#define DEPICT_ERROR_CAMERA_BUSY         DEPICT_MAKE_ERROR(11)
#define DEPICT_ERROR_DEVICE_BUSY         DEPICT_MAKE_ERROR(11)
#define DEPICT_ERROR_QUEUE               DEPICT_MAKE_ERROR(12)
#define DEPICT_ERROR_IMAGE_INVALID       DEPICT_MAKE_ERROR(30)
#define DEPICT_ERROR_IMAGE_FORMAT        DEPICT_MAKE_ERROR(31)
#define DEPICT_ERROR_IMAGE_TOO_SMALL     DEPICT_MAKE_ERROR(32)
#define DEPICT_ERROR_FILE_NOT_FOUND      DEPICT_MAKE_ERROR(40)
#define DEPICT_ERROR_FILE_WRITE_ERROR    DEPICT_MAKE_ERROR(41)
#define DEPICT_ERROR_FILE_FORMAT_INVALID DEPICT_MAKE_ERROR(42)
#define DEPICT_ERROR_CORRECTION_FORMAT   DEPICT_MAKE_ERROR(50)
#define DEPICT_ERROR_INVALID_VERSION     DEPICT_MAKE_ERROR(51)
#define DEPICT_ERROR_INVALID_PARAMETER   DEPICT_MAKE_ERROR(100)
#define DEPICT_ERROR_LUT_FILE_ERROR      DEPICT_MAKE_ERROR(103)

#define DEPICT_IS_SUCCESS(code) (code == DEPICT_SUCCESS)
#define DEPICT_IS_WARNING(code) ((code & 0xC0000000) == 0x80000000)
#define DEPICT_IS_ERROR(code)   ((code & 0xC0000000) == 0xC0000000)


/*******************
* Depict Functions *
*******************/

#ifdef __cplusplus

extern "C"
{

#endif


/************************
* Depict Version Reporting *
*************************/

DEPICT_RESULT _DEPICT_API __stdcall DepictGetDepictVersion
(
   ULONG*                      pnVersion,
   char*                       pszVersion,
   size_t                      maxLength
);


/*********************
* Device Enumeration *
*********************/

/**********************************************************************
   Causes Depict to attempt to locate and enumerate all of the Opteon
   devices attached to the specified platform type. Depending on the
   number of devices present, this call may take some time to complete.
   If ePlatformType is set to DepictPlatformTypeUnknown, then all
   platform types will be enumerated.

   arguments:
      ePlatformType - platform type to enumerate
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictEnumDevices
(
   DEPICT_PLATFORM_TYPE ePlatformType
);

/**********************************************************************
   Reports the number of cameras on the specified platform found by
   the last enumeration.

   arguments:
      ePlatformType - platform type of cameras. If this is set to
                      DepictPlatformTypeUnknown, the function returns
                      the total number of cameras on all platforms.
      pulNumEnumCameras - pointer to a ULONG that will be
                          filled with the number of cameras
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetNumEnumCameras
(
   DEPICT_PLATFORM_TYPE ePlatformType,
   ULONG * pulNumEnumCameras
);

/**********************************************************************
   Reports the number of controllers found by the last call to 
   DepictEnumDevices.

   arguments:
      ePlatformType - platform type of controllers. If this is set to
                      DepictPlatformTypeUnknown, the function returns
                      the total number of controllers on all platforms.
      pulNumEnumCameras - pointer to a ULONG that will be
                          filled with the number of cameras
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetNumEnumControllers
(
   DEPICT_PLATFORM_TYPE ePlatformType,
   ULONG * pulNumEnumControllers
);

/**********************************************************************
   Reports the basic information for a camera found by the last call
   to DepictEnumDevices.

   arguments:
      ePlatformType - platform type of camera. If this is set to
                      DepictPlatformTypeUnknown, the ulEnumCameraIndex
                      parameter indexes into the list of all cameras
                      on all platform types.
      ulEnumCameraIndex - index into the zero-indexed array
                          of enumerated cameras. 
      pulPlatformID - pointer to a ULONG that will be filled
                      with the camera's platform ID
      pulCameraID - pointer to a ULONG that will be filled
                    with the camera's camera ID
      pulSerialNumber - pointer to a ULONG that will be filled
                        with the camera's serial number. If the
                        serial number cannot be read for this
                        camera, the function will report
                        DEPICT_SERIAL_NUMBER_INVALID
      pbCameraOpen - pointer to a BOOL that will be filled
                     with a non-zero value (TRUE) if the
                     camera has already been opened by this
                     or another process, and with zero
                     (FALSE) otherwise.

      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetEnumCameraInfo
(
   DEPICT_PLATFORM_TYPE ePlatformType,
   ULONG ulEnumCameraIndex,
   ULONG * pulPlatformID,
   ULONG * pulCameraID,
   ULONG * pulSerialNumber,
   BOOL * pbCameraOpen
);

/**********************************************************************
   Reports the owner name for a camera found by the last call to 
   DepictEnumDevices. If the camera is not currently open, the
   owner name will be the empty string: "".

   arguments:
      ePlatformType - platform type of camera. If this is set to
                      DepictPlatformTypeUnknown, the ulEnumCameraIndex
                      parameter indexes into the list of all cameras
                      on all platform types.
      ulEnumCameraIndex - index into the zero-indexed array
                          of enumerated cameras. 
      strOwnerName - pointer to a string that will be filled
                     with the name of the "owner" that has
                     opened this camera. If the camera is not
                     open, then this string will be empty.
      ulMaxNameSize - should be set to the maximum length
                      string that may safely be placed in
                      strOwnerName.

      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetEnumCameraOwnerName
(
   DEPICT_PLATFORM_TYPE ePlatformType,
   ULONG ulEnumCameraIndex,
   LPSTR lpszOwnerName,
   ULONG ulMaxNameSize
);

/**********************************************************************
   Reports the basic information for a controller found by the last
   call to DepictEnumDevices.

   arguments:
      ePlatformType - platform type of controller. If this is set to
                      DepictPlatformTypeUnknown, the ulEnumContIndex
                      parameter indexes into the list of all controllers
                      on all platform types.
      ulEnumContIndex - index into the zero-indexed array
                        of enumerated controllers. 
      pulPlatformID - pointer to a ULONG that will be filled
                      with the controller's platform ID
      pulSerialNumber - pointer to a ULONG that will be filled
                        with the controller's serial number. If
                        the serial number cannot be read for
                        this controller, the function will report
                        DEPICT_SERIAL_NUMBER_INVALID
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetEnumControllerInfo
(
   DEPICT_PLATFORM_TYPE ePlatformType,
   ULONG ulEnumContIndex,
   ULONG * pulPlatformID,
   ULONG * pulSerialNumber
);

/**********************************************************************
   Sets the OwnerName that will be associated with cameras opened
   by this process.

   arguments:
      strOwnerName - pointer to a string containing the name
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error #
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetOwnerName
(
   LPSTR lpszOwnerName
);


/********************
* Device Open/Close *
********************/

/**********************************************************************
   Request by a user-mode process to gain control of a camera channel 
   by specifying a platform and camera ID.  The process returns a handle 
   that will be used as an exclusive key to the selected camera until it 
   is explicitly released. The connectted camera is set to factory 
   defaults.

   arguments:
      ulPlatformID - platform id 
      ulCameraID - camera id
      
   returns:
      camera handle on success
      DEPICT_INVALID_HANDLE on failure
**********************************************************************/
DEPICT_HANDLE _DEPICT_API __stdcall DepictOpenCamera
(
   ULONG     ulPlatformID,
   ULONG     ulCameraID
);

/**********************************************************************
   Request by a user-mode process to gain control of a camera by
   specifying the camera's serial number. The process returns a handle 
   that will be used as an exclusive key to the selected camera until it 
   is explicitly released. The connected camera is set to factory 
   defaults.

   This function uses information from the last call to
   DepictEnumDevices to find the camera. If this function has not been
   called since the camera was attached, or has only been called to
   enumerate platform types other than the one on which the camera
   resides, then the camera will not be found. In addition, this
   function will not be able to locate Classic cameras.

   arguments:
      ulSerialNumber - camera's serial number
      
   returns:
      camera handle on success
      DEPICT_INVALID_HANDLE on failure
**********************************************************************/
DEPICT_HANDLE _DEPICT_API __stdcall DepictOpenCameraBySerialNumber
(
   ULONG     ulSerialNumber
);

/**********************************************************************
   Request by a user-mode process to gain control of a controller 
   by specifying a platform ID.  The process returns a handle that
   will be used as an exclusive key to the controller  until it 
   is explicitly released.

   arguments:
      ulPlatformID - platform id 
  
   returns:
      controller handle on success
      NULL on failure
**********************************************************************/
DEPICT_HANDLE _DEPICT_API __stdcall DepictOpenController
(
   ULONG     ulPlatformID
);


/**********************************************************************
   Request by a user-mode process to release control of a device.
   All actions in progress are terminated without regard to their current 
   status. 
 
   arguments:
      hDevice - device handle
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCloseDevice
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   NOTE: This is now a legacy function. DepictCloseDevice() should
         be used instead

   Request by a user-mode process to released control of a camera channel.
   All actions in progress are terminated without regard to their current 
   status. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCloseCamera
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Request by a user-mode process to release all devices currently opened
   by the process. 
 
   arguments:
      None
   
   returns:
      DEPICT_SUCCESS on success
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCloseAllDevices
(
   void
);


/***************************
* Device ID Info Retrieval *
***************************/

/**********************************************************************
   Retrieves the platform ID of the device. 
 
   arguments:
      hDevice - device handle
      pulPlatformID - pointer to ULONG to be filled with ID
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetPlatformID
(
   DEPICT_HANDLE  hDevice,
   ULONG * pulPlatformID
);

/**********************************************************************
   Retrieves the camera ID of the device (if any). 
 
   arguments:
      hDevice - device handle
      pulCameraID - pointer to ULONG to be filled with ID
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraID
(
   DEPICT_HANDLE  hDevice,
   ULONG * pulCameraID
);

/**********************************************************************
   Retrieves the serial number of the device (if supported). 
 
   arguments:
      hDevice - device handle
      pulSerialNumber - pointer to ULONG to be filled with serial #
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraSerialNumber
(
   DEPICT_HANDLE  hDevice,
   ULONG * pulSerialNumber
);


/***********************************
* H/W and Driver Version Reporting *
************************************/

DEPICT_RESULT _DEPICT_API __stdcall DepictGetDeviceDriverVersion
(
   DEPICT_HANDLE  hDevice,
   USHORT * pusMajorVer,
   USHORT * pusMinorVer
);


/******************************
* Camera Capability Reporting *
******************************/

/**********************************************************************
   Retrieves the sensor's color capabilities. 
 
   arguments:
      hDevice - device handle
      peSensorColor - pointer to DEPICT_CAMERA_SENSOR_COLOR to be
                      filled with sensor color
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraSensorColor
(
   DEPICT_HANDLE  hDevice,
   DEPICT_CAMERA_SENSOR_COLOR * peSensorColor
);

/**********************************************************************
   Retrieves the sensor type. 
 
   arguments:
      hDevice - device handle
      peSensorType - pointer to DEPICT_CAMERA_SENSOR to be
                     filled with sensor type
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraSensorType
(
   DEPICT_HANDLE  hDevice,
   DEPICT_CAMERA_SENSOR * peSensorType
);

/**********************************************************************
   Retrieves the max ROI height and width. 
 
   arguments:
      hDevice - device handle
      pulMaxWidth - pointer to ULONG to be filled with max width
      pulMaxHeight - pointer to ULONG to be filled with max height
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraMaxROI
(
   DEPICT_HANDLE  hDevice,
   ULONG * pulMaxWidth,
   ULONG * pulMaxHeight
);

/**********************************************************************
   Retrieves the max bits-per-pixel for this camera. Note that for
   color cameras, this function reports the max bits per color channel.
 
   arguments:
      hDevice - device handle
      pucBitsPerPixel - pointer to UCHAR to be filled with max BPP
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraMaxBPP
(
   DEPICT_HANDLE  hDevice,
   UCHAR * pucBitsPerPixel
);

/**********************************************************************
   Retrieves the number of camera taps.
 
   arguments:
      hDevice - device handle
      pucTaps - pointer to UCHAR to be filled with number of taps
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraNumTaps
(
   DEPICT_HANDLE  hDevice,
   UCHAR * pucTaps
);

/**********************************************************************
   Retrieves a boolean value indicating whether an Optiport is present
   on the device.
 
   arguments:
      hDevice - device handle
      pbOptiportPresent - pointer to BOOL to be filled with a non-zero
                          value (TRUE) if an Optiport is present, and
                          with zero (FALSE) otherwise
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictIsOptiportPresent
(
   DEPICT_HANDLE  hDevice,
   BOOL * pbOptiportPresent
);


/**********************************************************************
   Retrieves the camera bundles present.
 
   arguments:
      hDevice - device handle
      pbAgility - pointer to BOOL to be filled with a non-zero value
                  (TRUE) if the Agility bundle is present, and with
                  zero (FALSE) otherwise
      pbFidelity - pointer to BOOL to be filled with a non-zero value
                   (TRUE) if the Fidelity bundle is present, and with
                   zero (FALSE) otherwise
      pbTrigger - pointer to BOOL to be filled with a non-zero value
                  (TRUE) if the Trigger bundle is present, and with
                  zero (FALSE) otherwise
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraBundles
(
   DEPICT_HANDLE  hDevice,
   BOOL * pbAgility,
   BOOL * pbFidelity,
   BOOL * pbTrigger
);

/**********************************************************************
   Retrieves a value containing bit flags that indicate which special
   camera functions are present.
 
   arguments:
      hDevice - device handle
      pulFunctionFlags - pointer to ULONG to be filled with camera
                         function flags
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraFunctions
(
   DEPICT_HANDLE  hDevice,
   ULONG * pulFunctionFlags
);

/**********************************************************************
   Retrieves the expected frame rate when acquiring full-ROI images
   with this camera.
 
   arguments:
      hDevice - device handle
      pulFullFrameRate - pointer to ULONG to be filled with the
                         expected frame rate
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraFullFrameRate
(
   DEPICT_HANDLE  hDevice,
   ULONG * pulFullFrameRate
);

/**********************************************************************
   Retrieves the range of valid gain values for this camera.
 
   arguments:
      hDevice - device handle
      pfMinGain - pointer to float to be filled with min gain
      pfMaxGain - pointer to float to be filled with max gain
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraGainRange
(
   DEPICT_HANDLE  hDevice,
   float * pfMinGain,
   float * pfMaxGain
);

/**********************************************************************
   Retrieves the range of valid offset values for this camera.
 
   arguments:
      hDevice - device handle
      pfMinOffset - pointer to float to be filled with min offset
      pfMaxOffset - pointer to float to be filled with max offset
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraOffsetRange
(
   DEPICT_HANDLE  hDevice,
   float * pfMinOffset,
   float * pfMaxOffset
);


/***************************
* Image Utility Prototypes *
***************************/

/**********************************************************************
   Allocates a contiguous region of user-mode virtual address space
   large enough to contain an image of the specified format with
   width = ulWidth and height = ulHeight.

   If possible, the function will create a buffer that is locked
   into physical memory. See DepictCanLockMemory.

   no restrictions apply as to when this function can be called. 
  
   arguments:
      ulWidth - number of pixels in a row of the image
      ulHeight - number of rows pixels in the image
      eFormat - desired image format
    
   returns:
      pointer to image buffer (size = width * height)
      NULL on error
**********************************************************************/
void _DEPICT_API * __stdcall DepictCreateImageDestinationEx
(
   ULONG                 ulWidth, 
   ULONG                 ulHeight,
   DEPICT_FORMAT         eFormat
);

/**********************************************************************
   NOTE: This is now a legacy function, and its use is not recommended.
         DepictCreateImageDestinationEx (see above) should be used
         instead.

   Allocates a contiguous region of user-mode virtual address space of 
   size = width * height.
   If possible, the function will create a buffer that is locked
   into physical memory. See DepictCanLockMemory.

   no restrictions apply as to when this function can be called. 
  
   arguments:
      width - number of pixels in a row of the image
      height - number of rows pixels in the image
    
   returns:
      pointer to image buffer (size = width * height)
      NULL on error
**********************************************************************/
void _DEPICT_API * __stdcall DepictCreateImageDestination
(
   ULONG                 width, 
   ULONG                 height
);

/**********************************************************************
   Frees a image region allocated with DepictCreateImageDestination().
 
   no restrictions apply as to when this function can be called. 
   
   arguments:
      pImage - pointer to user allocated image buffer (size = width * height)
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictFreeImageDestination
(
   void*                 pImage
);


/**********************************************************************
   Tells the user if buffers created with DepictCreateImageDestination
   will be locked into physical memory
 
   no restrictions apply as to when this function can be called. 
   
   arguments:
      None
     
   returns:
      zero if buffers will not be locked
      non-zero if they will be locked
**********************************************************************/
BOOL _DEPICT_API __stdcall DepictCanLockMemory(void);


/******************
* Image Load/Save *
*******************/

/**********************************************************************
   Stores ane image at a given destination as a BMP file at the 
   designated path/filespec.  

   arguments:
      pImage - pointer to user allocated image buffer (size = width * height)
      pszName - path and filename to store image in
      width - number of pixels in a row of the image
      height - number of rows pixels in the image
      bOverwrite - true if any previous stored file should be overwritten
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSaveImage
(
   void *   pvImage, 
   char *   pszName,
   ULONG    ulWidth, 
   ULONG    ulHeight,
   BOOL     bOverwrite
);


/**********************************************************************
   Stores ane image at a given destination as a BMP file at the 
   designated path/filespec.  
 
   arguments:
      pImage - pointer to user allocated image buffer (size = width * height)
      pszName - path and filename to store image in
      width - number of pixels in a row of the image
      height - number of rows pixels in the image
      bOverwrite - true if any previous stored file should be overwritten
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSaveColorImage
(
   void *   pImage, 
   char *   pszName,
   ULONG    ulWidth, 
   ULONG    ulHeight,
   BOOL     bOverwrite
);


/**********************************************************************
   Stores ane image at a given destination as a BMP file at the 
   designated path/filespec.  
 
   arguments:
      pImage - pointer to user allocated image buffer (size = width * height)
      pszName - path and filename to store image in
      width - number of pixels in a row of the image
      height - number of rows pixels in the image
      bOverwrite - true if any previous stored file should be overwritten
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSave32BitColorImage
(
   void *   pvImage, 
   char *   pszName,
   ULONG    ulWidth, 
   ULONG    ulHeight,
   BOOL     bOverwrite
);


/**********************************************************************
   Loads an BMP designated path/filespec to a given destination. 
 
   arguments:
      pImage - pointer to user allocated image buffer (size = width * height)
      pszName - path and filename to store image in
      width - number of pixels in a row of the image
      height - number of rows pixels in the image
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictLoadImage
(
   void *   pvImage, 
   char *   pszName,
   ULONG    ulWidth, 
   ULONG    ulHeight
);


/**********************************************************************
   Loads a color BMP designated path/filespec to a given destination. 
 
   arguments:
      pImage - pointer to user allocated image buffer (size = width * height)
      pszName - path and filename to store image in
      ulWidth - number of pixels in a row of the image
      ulHeight - number of rows pixels in the image
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictLoadColorImage
(
   void *   pvImage, 
   char *   pszName,
   ULONG    ulWidth, 
   ULONG    ulHeight
);


/**********************************************************************
   Retrieves information about and image on disk. 
 
   arguments:
      pszName - path and filename to store image in
      pulWidth - number of pixels in a row of the image
      pulHeight - number of rows pixels in the image
      peFormat - image format
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictLoadImageInfo
(
   char *            pszName,
   ULONG *           pulWidth, 
   ULONG *           pulHeight,
   DEPICT_FORMAT *   peFormat
);
/* End Image Utility Prototypes.
 
   
/******************
* Controller Ops  *
*******************/     
       
/**********************************************************************
   Resets the Controller.  Resets all cameras connected to the specified 
   camera's controller.  Deletes all service requests pending. The 
   connected cameras is set to the most recently sent parameters. 
         
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
           
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictResetController
(
 DEPICT_HANDLE    hDevice 
 );


 /**********************************************************************
 Resets the Controller specified. you need to give
 a valid camera number as well, but the camera is
 not reset
 
   arguments:
   board - board ID
   camera - camera ID
   
     returns:
     DEPICT_SUCCESS on success
     !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
     DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictResetControllerFromID
(
 ULONG       board,
 ULONG       camera
 );


 /**********************************************************************
 Returns the type of controller. 
 
   arguments:
   hDevice - board camera handle returned by DepictOpenCamera()
   
     returns:
     a valid ULONG
     or ULONG_UNKNOWN
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetControllerType
(
 DEPICT_HANDLE    hDevice 
 );

   
/******************
* Camera Ops  *
*******************/     

/**********************************************************************
   Resets the specified camera connected and deletes all service requests 
   pending. The connected camera is set to the camera defaults.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictResetCamera
(
   DEPICT_HANDLE    hDevice 
);


/**********************************************************************
   Resets and pwers down the specified camera connected.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCameraOff
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Clears all pending acquisitions. Any acquisition in progress which is 
   allowed to go to completion. The connected camera is set to the 
   acquistion settings of the most recent acquisition. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictClearCameraQueue
(
   DEPICT_HANDLE    hDevice 
);


/**********************************************************************
   Clears all pending acquisitions on all opened cameras. Any acquisition 
   in progress which is allowed to go to completion. The connected cameras 
   are set to the acquistion settings of the most recent acquisition. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictClearAllCameraQueues();


/**********************************************************************
   NOTE: DepictSingleAcquire is a legacy function, and its use is not
         recommended. DepictSingleAcquireEx (see below) should be used
         instead.

   Places a service request on the queue for the designated camera to 
   return a image structure to a user-mode virtual memory area 
   previously allocated.  The acquisition begins immediately if the 
   trigger mode for the designated camera is "immediate," and an 
   acquisition is not already in progress on the designated camera.  

   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pImage - pointer to user allocated image buffer (size = width * height)
      eTriggerMode - a legal trigger enumeration:
                       DepictTriggerImmediate,   /* indicates immediate trigger
                       DepictTriggerTimebase,    /* indicates timebase trigger
                        DepictTriggerExternalA,   /* indicates external trigger A
                        DepictTriggerExternalB,   /* indicates external trigger B
                        DepictTriggerMaster,      /* indicates external master trigger
                        DepictTriggerROI          /* indicates trigger on ROI
      hEvent - a user created event (CreateEvent) which is set on image 
               available. if NULL this function waits to completion or 10 secs. 
               The event must be reset by the caller. 
      pTimestamp - a pointer if non-NULL it will be filled with pentium clock tick
                   at the time of trigger. See WIN32 API  QueryPerformanceCounter() 
                   & QueryPerformanceFrequency().
  
   notes:
      this function is queued behind other queued operations to this camera
      unless hEvent is NULL and then it is only allowed if operations are 
      not queued
    
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_CAMERA_BUSY if acquisitions are queued & hEvent is NULL
      WAIT_TIMEOUT if hEvent is NULL and 10 seconds have expired
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSingleAcquire
(
   DEPICT_HANDLE   hDevice,
   DEPICT_TRIGGER  eTriggerMode,
   void*           pvImage,
   HANDLE          hEvent,
   HANDLE          hTriggerEvent,
   LARGE_INTEGER * pTriggerTimestamp,
   void *          pvRedundant,
   ULONG           ulStride
);


/**********************************************************************
   Places a service request on the queue for the designated camera to 
   return a image structure to a user-mode virtual memory area 
   previously allocated.  The acquisition begins immediately if the 
   trigger mode for the designated camera is "immediate," and an 
   acquisition is not already in progress on the designated camera.  

   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      eTriggerMode - a legal trigger enumeration:
                       DepictTriggerImmediate,   /* indicates immediate trigger
                       DepictTriggerTimebase,    /* indicates timebase trigger
                       DepictTriggerExternalA,   /* indicates external trigger A
                       DepictTriggerExternalB,   /* indicates external trigger B
                       DepictTriggerMaster,      /* indicates external master trigger
                       DepictTriggerROI          /* indicates trigger on ROI
      pImage - pointer to user allocated image buffer (size = width * height)
      psAcqStatus - pointer to an initialized DEPICT_ACQ_STATUS structure.
                    This structure can contain an acquisition complete
                    event, trigger event, and trigger timestamp. If the
                    pointer passed is NULL, the function will do a
                    blocking grab.
  
   notes:
      this function is queued behind other queued operations to this camera
      unless hEvent is NULL and then it is only allowed if operations are 
      not queued
    
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_CAMERA_BUSY if acquisitions are queued & hEvent is NULL
      WAIT_TIMEOUT if hEvent is NULL and 10 seconds have expired
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSingleAcquireEx
(
   DEPICT_HANDLE       hDevice,
   DEPICT_TRIGGER      eTriggerMode,
   void*               pvImage,
   DEPICT_ACQ_STATUS * psAcqStatus,
   void *              pvRedundant,
   ULONG               ulStride
);


/**********************************************************************
   Sets the desired image format for acquisitions. 
 
   arguments:
      hDevice - camera handle returned by DepictOpenCamera()
      eFormat - desired format

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetImageFormat
(
   DEPICT_HANDLE hDevice,
   DEPICT_FORMAT eFormat
);


/**********************************************************************
   Gets the current image format for acquisitions.
 
   arguments:
      hDevice - camera handle returned by DepictOpenCamera()
      eFormat - pointer to be filled with current format

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetImageFormat
(
   DEPICT_HANDLE hDevice,
   DEPICT_FORMAT *        peFormat
);


/**********************************************************************
   Sets the desired image format for redundant display transfers. 
 
   arguments:
      hDevice - camera handle returned by DepictOpenCamera()
      eFormat - desired format

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetDisplayFormat
(
   DEPICT_HANDLE hDevice,
   DEPICT_FORMAT eFormat
);


/**********************************************************************
   Gets the current image format for redundant display transfers. 
 
   arguments:
      hDevice - camera handle returned by DepictOpenCamera()
      eFormat - pointer to be filled with current format

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetDisplayFormat
(
   DEPICT_HANDLE hDevice,
   DEPICT_FORMAT * peFormat
);


/**********************************************************************
   Checks the status of an acquisition in progress using the
   DEPICT_ACQ_STATUS structure passed to a DepictSingleAcquireEx
   call. In addition to determining whether the acquisition has
   returned or not, this function can also return any error code
   associated with an acquisition.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      psAcqStatus - the DEPICT_ACQ_STATUS structure passed to 
                    DepictSingleAcquireEx().
      nTimeoutMSEC - time to wait till done in milliseconds
                     use the WIN32 defined INFINITE to wait forever
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_TIMEOUT on timeout before acquire finished
      WIN32 GetLastError() on any other error
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictWaitAcquireCompleteEx
(
   DEPICT_HANDLE       hDevice,
   DEPICT_ACQ_STATUS * psAcqStatus,
   ULONG               ulMilliseconds
);


/**********************************************************************
   NOTE: This is a legacy function, and its use is not recommended.
         DepictWaitAcquireCompleteEx (see above) should be used
         instead.

   Checks the status of an acquisition in progress. 
 
   arguments:
      hEvent - a user created event (CreateEvent) which was give to 
               DepictSingleAcquire().
      nTimeoutMSEC - time to wait till done in milliseconds
                     use the WIN32 defined INFINITE to wait forever
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_TIMEOUT on timeout before acquire finished
      WIN32 GetLastError() on any other error
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictWaitAcquireComplete
(
   HANDLE hEvent,
   ULONG  ulTimeoutMSEC
);


/**********************************************************************
   Returns the trigger status.. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pulStatus - trigger status word
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetExternalTriggerStatus
(
   DEPICT_HANDLE hDevice,
   ULONG *       pulStatus
);


/**********************************************************************
   Forces the use of compensation tables when interpreting the results 
   from the designated camera.  
 
   notes:
      this function is unqueued and not allowed during queued operations
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pPixelCompensation - pointer to pixel compensation array (size = width * height)
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values in map file
      DEPICT_ERROR_CAMERA_BUSY if called when acquisitions are queued
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictLoadCameraCompensationFromFile
(
   DEPICT_HANDLE hDevice,
   const char *  pszFilename
);


/**********************************************************************
   Forces the use of compensation tables when interpreting the results 
   from the designated camera.  
 
   arguments:
   hDevice - board camera handle returned by DepictOpenCamera()
   
   notes:
      this function is queued behind other queued operations to this camera
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictEnablePixelCompensation
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Disables the use of compensation tables when interpreting the results 
   from the designated camera.  
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictDisablePixelCompensation
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Determines whether pixel compensation tables are in use with the 
   designated camera.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pbEnabled - a pointer to a BOOL to be set on return
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictIsEnabledPixelCompenstation
(
   DEPICT_HANDLE  hDevice,
   BOOL *               pbEnabled
);


/**********************************************************************
   Creates a compensation table file based on the current static image.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pszFilename - the filename to store the comenstation table
      ulXStart - pixel column to start acquisition on (0 is the first pixel)
      ulXEnd - pixel column to end acquisition on (max depends on camera)
      ulYStart - line to start acquisition on (0 is the first line)
      ulYEnd - line to end acquisition on (max depends on camera)
      bOverwrite - true if any previous stored file should be overwritten
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictRecalibrateCamera
(
   DEPICT_HANDLE  hDevice, 
   const char*    pszFilename,
   ULONG          ulXStart,
   ULONG          ulYStart,
   ULONG          ulXEnd,
   ULONG          ulYEnd,
   BOOL           bOverwrite
);


/**********************************************************************
   Creates a compensation table file based on the current static image
   for a camera that produces a Bayer-pattern image. The only difference
   between this function and DepictRecalibrateCamera is that this
   function calculates the gains and offsets separatley for red,
   green, and blue pixels.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pszFilename - the filename to store the comenstation table
      ulXStart - pixel column to start acquisition on (0 is the first pixel)
      ulXEnd - pixel column to end acquisition on (max depends on camera)
      ulYStart - line to start acquisition on (0 is the first line)
      ulYEnd - line to end acquisition on (max depends on camera)
      bOverwrite - true if any previous stored file should be overwritten
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictRecalibrateBayerColorCamera
(
   DEPICT_HANDLE  hDevice, 
   const char*    pszFilename,
   ULONG          ulXStart,
   ULONG          ulYStart,
   ULONG          ulXEnd,
   ULONG          ulYEnd,
   BOOL           bOverwrite
);


/**********************************************************************
   Defines the rectangular region of interest that will be written to 
   user-mode virtual memory as a result of an acquisition. 
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulLeft - pixel column to start acquisition on (0 is the first pixel)
      ulTop - pixel row to start acquisition on (0 is the first row)
      ulWidth - width of ROI
      ulHeight - height of ROI
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetPrimaryROIEx
(
   DEPICT_HANDLE  hDevice,
   ULONG ulLeft,
   ULONG ulTop,
   ULONG ulWidth,
   ULONG ulHeight
);


/**********************************************************************
   Used to determine the rectangular region of interest that will be 
   written to user-mode virtual memory as a result of an acquisition. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pulLeft - pointer to pixel column to start acquisition on (0 is the first pixel)
      pulTop - pointer pixel row to start acquisition on (0 is the first row)
      pulWidth - pointer to width of ROI
      pulHeight - pointer height of ROI
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetPrimaryROIEx
(
   DEPICT_HANDLE  hDevice, 
   ULONG * pulLeft,
   ULONG * pulTop,
   ULONG * pulWidth,
   ULONG * pulHeight
);


/**********************************************************************
   Defines the rectangular region of interest that will be written to 
   user-mode virtual memory as a result of an acquisition. 
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      xStart - pixel column to start acquisition on (0 is the first pixel)
      xEnd - pixel column to end acquisition on (max depends on camera)
      yStart - line to start acquisition on (0 is the first line)
      yEnd - line to end acquisition on (max depends on camera)
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetPrimaryROI
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulXStart,
   ULONG          ulYStart,
   ULONG          ulXEnd,
   ULONG          ulYEnd
);


/**********************************************************************
   Used to determine the rectangular region of interest that will be 
   written to user-mode virtual memory as a result of an acquisition. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pxStart - a pointer to be filled with pixel column to start acquisition
      pxEnd - a pointer to be filled with pixel column to end acquisition
      pyStart - a pointer to be filled with line to start acquisition 
      pyEnd - a pointer to be filled with line to end acquisition on 
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetPrimaryROI
(
   DEPICT_HANDLE  hDevice, 
   ULONG*         pulXStart,
   ULONG*         pulYStart,
   ULONG*         pulXEnd,
   ULONG*         pulYEnd
);


/**********************************************************************
   Defines the exposure time that will be used to generate the image that 
   will be written to user-mode virtual memory as a result of subsequent 
   acquisitions.  Production cameras are programmable from 50 sec to 
   100,000,000 secs with a resolution of 50 sec.  Exposures are 
   expressed in units of microseconds.    
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      nExposure - acquisition exposure time in microseconds
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetPrimaryExposure
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulExposure
);


/**********************************************************************
   Determines the current exposure time.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pnExposure - a pointer to be filled with exposure time in microseconds
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetPrimaryExposure
(
   DEPICT_HANDLE hDevice,
   ULONG*        pulExposure
);


/**********************************************************************
   Defines the strobe logic for the camera.    
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      eStrobeLogic - the desired strobe logic
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetStrobeLogic
(
   DEPICT_HANDLE hDevice,
   DEPICT_STROBE eStrobeLogic
);


/**********************************************************************
   Determines the current strobe logic for the camera. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      peStrobeLogic - pointer to value to be filled
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetStrobeLogic
(
   DEPICT_HANDLE   hDevice,
   DEPICT_STROBE * peStrobeLogic
);


/**********************************************************************
   Defines the gain for the exposure.    
 
   notes:
      this function is queued behind other queued operations to this
      camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      fGain - gain
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetCameraGain
(
   DEPICT_HANDLE hDevice,
   float         fGain
);

/**********************************************************************
   Determines the current camera gain.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pfGain - a pointer to be filled with gain
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraGain
(
   DEPICT_HANDLE hDevice,
   float *       pfGain
);


/**********************************************************************
   Defines the offset for the exposure.    
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      fOffset - offset
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetCameraOffset
(
   DEPICT_HANDLE hDevice,
   float         fOffset
);


/**********************************************************************
   Determines the current camera offset.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pfOffset - a pointer to be filled with offset
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraOffset
(
   DEPICT_HANDLE hDevice,
   float *       pfOffset
);


/**********************************************************************
   Defines the gain for the exposure.    
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulChannel - channel to which to apply the change
      fGain - gain
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetCameraGainEx
(
   DEPICT_HANDLE hDevice,
   ULONG         ulChannel,
   float         fGain
);

/**********************************************************************
   Determines the current camera gain.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulChannel - channel to which to apply the change
      pfGain - a pointer to be filled with gain
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraGainEx
(
   DEPICT_HANDLE hDevice,
   ULONG         ulChannel,
   float *       pfGain
);


/**********************************************************************
   Defines the offset for the exposure.    
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulChannel - channel to which to apply the change
      fOffset - offset
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetCameraOffsetEx
(
   DEPICT_HANDLE hDevice,
   ULONG         ulChannel,
   float         fOffset
);


/**********************************************************************
   Determines the current camera offset.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulChannel - channel to which to apply the change
      pfOffset - a pointer to be filled with offset
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraOffsetEx
(
   DEPICT_HANDLE hDevice,
   ULONG         ulChannel,
   float *       pfOffset
);


/**********************************************************************
   Defines the # of lines which will be binned together.
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulBins = ULONG # of lines (currently 1, 2, 3, or 4 are supportted)
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetPrimaryBinning
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulBins
);


/**********************************************************************
   Determines the current camera binning value.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pulBins - a pointer to be filled with the number of bins
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetPrimaryBinning
(
   DEPICT_HANDLE  hDevice,
   ULONG *        pulBins
);


/**********************************************************************
   Enables the tracking of camera busy or "missed trigger" counting.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictEnableCameraBusyMonitor
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Disables the tracking of camera busy or "missed trigger" counting.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictDisableCameraBusyMonitor
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Returns the number of camera busy or "missed trigger" count since it 
   was enabled.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetCameraBusyCount
(
   DEPICT_HANDLE  hDevice,
   ULONG *        pulCount
);


/**********************************************************************
   Defines the rectangular region of interest for the trigger AOI.
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      xStart - pixel column to start acquisition on (0 is the first pixel)
      xEnd - pixel column to end acquisition on (max depends on camera)
      yStart - line to start acquisition on (0 is the first line)
      yEnd - line to end acquisition on (max depends on camera)
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetAOITriggerROI
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulXStart,
   ULONG          ulYStart,
   ULONG          ulXEnd,
   ULONG          ulYEnd
);

/**********************************************************************
   Used to determine the rectangular region of interest that will be used 
   for the AOI trigger.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pxStart - a pointer to be filled with pixel column to start acquisition
      pxEnd - a pointer to be filled with pixel column to end acquisition
      pyStart - a pointer to be filled with line to start acquisition 
      pyEnd - a pointer to be filled with line to end acquisition on 
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetAOITriggerROI
(
   DEPICT_HANDLE  hDevice, 
   ULONG*         pulXStart,
   ULONG*         pulYStart,
   ULONG*         pulXEnd,
   ULONG*         pulYEnd
);


/**********************************************************************
   Defines the exposure time that will be used to generate the trigger 
   AOI image. Production cameras are programmable from 50 sec to 
   100,000,000 secs with a resolution of 50 sec.  Exposures are 
   expressed in units of microseconds.    
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      nExposure - acquisition exposure time in microseconds
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetAOITriggerExposure
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulExposure
);


/**********************************************************************
   Determines the current exposure time for the AOI trigger.  
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pnExposure - a pointer to be filled with exposure time in microseconds
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetAOITriggerExposure
(
   DEPICT_HANDLE  hDevice,
   ULONG*         pulExposure
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      typeThres
      typeDeviation
      lower - a value in pixel intensity 0 - 255
      upper - a value in pixel intensity 0 - 255
      stddev - a value in pixel intensity 0 - 255
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetAOITriggerThresholds
(
   DEPICT_HANDLE                 hDevice,
   DEPICT_THRESHOLD              eThreshType,
   DEPICT_DEVIATION_REQUIREMENT  eDevReq,
   ULONG                         ulThresh1,
   ULONG                         ulThresh2,
   ULONG                         ulStdDev
);


/**********************************************************************
   Used to determine the current AOI trigger specification. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ptypeThres
      ptypeDeviation
      plower
      pupper
      pstddev
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetAOITriggerThresholds
(
   DEPICT_HANDLE                 hDevice,
   DEPICT_THRESHOLD*             peThreshType,
   DEPICT_DEVIATION_REQUIREMENT* peDevReq,
   ULONG*                        pulThresh1,
   ULONG*                        pulThresh2,
   ULONG*                        pulStdDev
);


/**********************************************************************
   Sets the AOI Trigger Holdoff, in microseconds.

   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulHoldoff - holdoff from trigger to acquisition start in microseconds
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetAOITriggerHoldoff
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulHoldoff
);


/**********************************************************************
   Used to determine the current value for the AOI Trigger Holdoff
   in microseconds. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pnUSECS - a pointer to holdoff value in microseconds
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetAOITriggerHoldoff
(
   DEPICT_HANDLE  hDevice,
   ULONG *        pulHoldoff
);


/**********************************************************************
   This function specifies the minimium time between timebase acquisitions
   (DepictTriggerTimebase trigger mode). 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      nPeriod - acquisition repeatition period in microseconds
   
   notes:
      this function is queued behind other queued operations to this camera
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetTriggerPeriod
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulPeriod
);


/**********************************************************************
   Used to determine the current value for Trigger Period. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pnPeriod - a pointer to acquisition repeatition period in microseconds
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetTriggerPeriod
(
   DEPICT_HANDLE  hDevice,
   ULONG *        pulPeriod
);


/**********************************************************************
   This function specifies a delay between a hardware trigger occuring and
   the camera starting the acquisition process. Since the delay only
   applies to external triggers, it is only used for acquisitions requested
   with the DepictTriggerExternalA, DepictTriggerExternalB, or
   DepictTriggerMaster trigger modes.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulDelay- acquisition delay in microseconds
   
   notes:
      this function is queued behind other queued operations to this camera
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetExternalTriggerDelay
(
   DEPICT_HANDLE  hDevice,
   ULONG          ulDelay
);

/**********************************************************************
   Used to determine the current value for the hardware trigger delay. 
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pulDelay - a pointer to acquisition delay in microseconds
   
   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_CAMERA_BUSY if acquistions are queued
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetExternalTriggerDelay
(
   DEPICT_HANDLE  hDevice,
   ULONG*         pulDelay
);

/**********************************************************************
   Sets the number of exposures to be performed for each acquisition.
   The default is 1. Any number above this will specify that the CCD
   should be exposed that many times before the image data is sent.
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ucMultiExposureCount - number of exposures to perform
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetMultiExposureCount
(
   DEPICT_HANDLE  hDevice,
   UCHAR ucMultiExposureCount
);

/**********************************************************************
   Gets the number of exposures to be performed for each acquisition.
   The default is 1. Any number above this indicates that the CCD
   should be exposed that many times before the image data is sent.
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pucMultiExposureCount - number of exposures that will be performed
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetMultiExposureCount
(
   DEPICT_HANDLE  hDevice,
   UCHAR * pucMultiExposureCount
);

/**********************************************************************
   Enables "pipelined" exposure, such that exposure for one 
   acquisition may begin while the data from the previous acquisition
   is being transferred to the host.
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetExposureToNonPipelined
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Disables "pipelined" exposure.
 
   notes:
      this function is queued behind other queued operations to this camera
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetExposureToPipelined
(
   DEPICT_HANDLE  hDevice
);


/**********************************************************************
   Determine if the camera is in "pipelined" exposure mode.
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pbPipelined - returns a non-zero value if exposure is pipelined,
                    zero otherwise
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictIsExposurePipelined
(
   DEPICT_HANDLE  hDevice,
   BOOL*          pbPipelined
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      ulCameraID - the camera to slave this camera to (master)
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid values
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetSlaveToChannel
(
   DEPICT_HANDLE hDevice,
   ULONG         ulCameraID 
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pCameraID - the camera to which this camera is slaved
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid pointer
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetSlaveToChannel
(
   DEPICT_HANDLE  hDevice,
   ULONG *        pulCameraID 
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      bRising - boolean value. If TRUE, trigger is on rising edge,
                if FALSE, trigger is on falling edge
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetMasterTriggerEdge
(
   DEPICT_HANDLE hDevice,
   BOOL bRising
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pbRising - pointer to boolean value. If TRUE, trigger is on rising edge,
                 if FALSE, trigger is on falling edge
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid pointer
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetMasterTriggerEdge
(
   DEPICT_HANDLE  hDevice,
   BOOL *         pbRising
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      bRising - boolean value. If TRUE, trigger is on rising edge,
                if FALSE, trigger is on falling edge
 
   notes:
   this function is queued behind other queued operations to this camera
   
     returns:
     DEPICT_SUCCESS on success
     !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
     DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetExternalTriggerEdge
(
   DEPICT_HANDLE hDevice,
   BOOL          bRising
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pbRising - pointer to boolean value. If TRUE, trigger is on rising edge,
                 if FALSE, trigger is on falling edge
 
   notes:
      this function is queued behind other queued operations to this camera
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is a DEPICT.DLL error # or WIN32 GetLastError()
      DEPICT_ERROR_NO_CAMERA_FOUND for camera not available
      DEPICT_ERROR_INVALID_PARAMETER for invalid pointer
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetExternalTriggerEdge
(
   DEPICT_HANDLE  hDevice,
   BOOL *         pbRising
);

/* End Camera interface prototypes.*/


/********************************
* OptiPort Inteface Prototypes. *
*********************************/
     
/**********************************************************************
   arguments:
      hDevice - handle to an Opteon device that supports OptiPort
      usAddress - OptiPort device address
      usOutputValue - value to send to device
      pulInputValue - pointer to memory to fill with 32-bit
                      output value
      psStatus - pointer to an initialized SIOStatus structure
                 that may be passed to DepictWaitForOptiPort.
                 If this is NULL, Depict will wait until the
                 transaction is complete before returning.

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictDoOptiPortTransaction
(
   DEPICT_HANDLE hDevice,
   USHORT usAddress,
   USHORT usOutputValue,
   ULONG * pulInputValue,
   DEPICT_IO_STATUS * psIOStatus
);
     
/**********************************************************************
   arguments:
      hDevice - handle to an Opteon device that supports OptiPort
      psStatus - pointer to SIOStatus structure that may was
                 passed to Depict call initiating an Optiport
                 operation
      ulMilliseconds - number of milliseconds to wait for
                       OptiPort operation to complete before
                       returning DEPICT_ERROR_TIMEOUT

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictWaitForOptiPort
(
   DEPICT_HANDLE hDevice,
   DEPICT_IO_STATUS * psIOStatus,
   ULONG ulMilliseconds
);


/***************************
* TTL Inteface Prototypes. *
***************************/
     
/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      usData - data to write to the TTL output register
       
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetDigitalPort
(
  DEPICT_HANDLE  hDevice,
  USHORT               usData
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pusData - pointer to data to read from the TTL input lines
 
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetDigitalPort
(
  DEPICT_HANDLE  hDevice,
  USHORT *             pusData
);


/**********************************************************************
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pusData - pointer to data to read from the TTL input lines
      psIOStatus - pointer to a DEPICT_IO_STATUS structure if (NULL
                   waits to completion). If non-NULL the user can
                   call DepictWaitLatchedDigitalIO to get the status
                   of the IO request and/or wait for it to complete.
                   hEvent is required to be a manual reset event
                   see CreateEvent().
 
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetLatchedDigitalPort
(
  DEPICT_HANDLE  hDevice,
  USHORT *             pusData,
  DEPICT_IO_STATUS *   psIOStatus
);

/**********************************************************************
   Waits for digital IO to become ready
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      psIOStatus - pointer to a DEPICT_IO_STATUS structure that was
                   passed to DepictGetLatchedDigitalIO
      ulMilliseconds - number of milliseconds to wait before returning
                       with a result of DEPICT_ERROR_TIMEOUT (this
                       does not cancel the IO request, and the function
                       may be called again to continue waiting).

   returns:
      DEPICT_SUCCESS on success
      DEPICT_ERROR_TIMEOUT on timeout
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictWaitLatchedDigitalIO
(
  DEPICT_CAMERA_HANDLE  hCamera,
  DEPICT_IO_STATUS *          psIOStatus,
  ULONG                       ulMilliseconds             
);

/**********************************************************************
   Sets controller's control tables
 
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
      pszFileName - name of file containing table data
   
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictSetControlTables
(
  DEPICT_HANDLE   hDevice,
  const char * pszFileName
);

/**********************************************************************
   Deactivates, or cancels, any active control tables on the controller
   
   arguments:
      hDevice - board camera handle returned by DepictOpenCamera()
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCancelControlTables
(
  DEPICT_HANDLE   hDevice
);

/* End TTL Inteface Prototypes.
   

/*****************************
* Image Processing Functions *
*****************************/

/**********************************************************************
   Rotates an image.
   
   arguments:
      pvSrcBuffer - pointer to source image buffer
      pvDestBuffer - pointer to source image buffer
      eFormat - image format
      ulWidth - width of source image
      ulHeight - height of source image
      dDegrees - degrees to rotate the image

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictRotateImage
(
   const void * pvSrcBuffer,
   void * pvDestBuffer,
   DEPICT_FORMAT eFormat,
   ULONG ulWidth,
   ULONG ulHeight,
   double dDegrees
);

/**********************************************************************
   Performs an in-place vertical mirroring
   
   arguments:
      pvBuffer - pointer to image buffer
      ulWidth - width of image
      ulHeight - height of image
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictMirrorVerticalInPlace
(
   void * pvBuffer,
   ULONG ulWidth,
   ULONG ulHeight
);

/**********************************************************************
   Performs a vertical mirroring
   
   arguments:
      pvSrcBuffer - pointer to image buffer
      pvDestBuffer - pointer to image buffer
      ulWidth - width of image
      ulHeight - height of image
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictMirrorVertical
(
   void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulWidth,
   ULONG ulHeight
);

/**********************************************************************
   Performs a vertical mirroring
   
   arguments:
      pvSrcBuffer - pointer to image buffer
      pvDestBuffer - pointer to image buffer
      ulWidth - width of image
      ulHeight - height of image
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictMirrorHorizontal
(
   void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulWidth,
   ULONG ulHeight
);

/**********************************************************************
   Converts mixed format image returned by camera into a
   standard 32-bit RGB format image.
       
   The color channels are weighted as follows:
        
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
           
   arguments:
      pBuffer - pointer to mixed format image buffer
      nMixedWidth - width of mixed format buffer
      nMixedHeight - height of mixed format buffer
             
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertMixedToRGB
(
   void * pBuffer,
   ULONG nMixedWidth,
   ULONG nMixedHeight
);
     
/**********************************************************************
   Converts grayscale image into a 24-bit RGB color image
           
   arguments:
      pSrcBuffer - pointer to bayer pattern image buffer
      pSrcBuffer - pointer to RGB24 image buffer
      nMixedWidth - width of buffer
      nMixedHeight - height of buffer
             
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertGrayscaleToRGB24
(
   const void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulWidth,
   ULONG ulHeight
);
     
/**********************************************************************
   Converts grayscale image into a 32-bit RGB color image
       
   The color channels are weighted as follows:
        
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
           
   arguments:
      pSrcBuffer - pointer to bayer pattern image buffer
      pSrcBuffer - pointer to RGB24 image buffer
      nMixedWidth - width of buffer
      nMixedHeight - height of buffer
             
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertGrayscaleToRGB32
(
   const void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulWidth,
   ULONG ulHeight
);
     
/**********************************************************************
   Converts Bayer-pattern image returned by camera into a
   standard grayscale image.
       
   The color channels are weighted as follows:
        
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
           
   arguments:
      pSrcBuffer - pointer to bayer pattern image buffer
      pSrcBuffer - pointer to grayscale image buffer
      nMixedWidth - width of buffer
      nMixedHeight - height of buffer
             
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertBayerToGrayscale
(
   void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulXStart,
   ULONG ulYStart,
   ULONG ulWidth,
   ULONG ulHeight
);
     
/**********************************************************************
   Converts Bayer-pattern image returned by camera into a
   standard 24-bit RGB format image.
       
   The color channels are weighted as follows:
        
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
           
   arguments:
      pSrcBuffer - pointer to bayer pattern image buffer
      pSrcBuffer - pointer to RGB24 image buffer
      nMixedWidth - width of buffer
      nMixedHeight - height of buffer
             
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertBayerToRGB24
(
   void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulXStart,
   ULONG ulYStart,
   ULONG ulWidth,
   ULONG ulHeight
);
     
/**********************************************************************
   Converts Bayer-pattern format image returned by camera into a
   standard 32-bit RGB format image.
       
   The color channels are weighted as follows:
        
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
           
   arguments:
      pSrcBuffer - pointer to bayer pattern image buffer
      pSrcBuffer - pointer to RGB24 image buffer
      nMixedWidth - width of buffer
      nMixedHeight - height of buffer
             
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertBayerToRGB32
(
   void * pvSrcBuffer,
   void * pvDestBuffer,
   ULONG ulXStart,
   ULONG ulYStart,
   ULONG ulWidth,
   ULONG ulHeight
);

/**********************************************************************
   Converts mixed format image returned by camera
   into 3 separate images, one for each colour channel.
   
   The color channels are weighted as follows:
     
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
       
   arguments:
      pBuffer - pointer to mixed format image buffer
      nMixedWidth - width of mixed format buffer
      nMixedHeight - height of mixed format buffer
      pRedBuffer - pointer to red channel buffer
      pGreenBuffer - pointer to green channel buffer
      pBlueBuffer - pointer to blue channel buffer
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertBayerToRGBSeparate
(
	void * pvSrcBuffer,
	void * pvRedBuffer,
	void * pvGreenBuffer,
	void * pvBlueBuffer,
   ULONG ulXStart,
   ULONG ulYStart,
   ULONG ulWidth,
   ULONG ulHeight
);

/**********************************************************************
   Converts mixed format image returned by camera
   into 3 separate images, one for each colour channel.
   
   The color channels are weighted as follows:
     
   Red   : 1.25
   Green : 1.0
   Blue  : 1.5
       
   arguments:
      pBuffer - pointer to mixed format image buffer
      nMixedWidth - width of mixed format buffer
      nMixedHeight - height of mixed format buffer
      pRedBuffer - pointer to red channel buffer
      pGreenBuffer - pointer to green channel buffer
      pBlueBuffer - pointer to blue channel buffer
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertMixedToRGBSeparate
(
   void * pBuffer,
   ULONG nMixedWidth,
   ULONG nMixedHeight,
   void * pRedBuffer,
   void * pGreenBuffer,
   void * pBlueBuffer
);

   /**********************************************************************
   Converts mixed format image returned by camera into a standard
   32-bit RGB format image, with user-supplied color channel weights.
   
     arguments:
     pBuffer - pointer to mixed format image buffer
     nMixedWidth - width of mixed format buffer
     nMixedHeight - height of mixed format buffer
     fRedWeight - red channel weight
     fGreenWeight - green channel weight
     fBlueWeight - blue channel weight
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertMixedToRGBWeighted
(
   void * pBuffer,
   ULONG nMixedWidth,
   ULONG nMixedHeight,
   float fRedWeight,
   float fGreenWeight,
   float fBlueWeight
   );

   /**********************************************************************
   Gets statistics for the three color channels of an RGB image.
   
     arguments:
     pColorBuffer - pointer to RGB image buffer
     nWidth - width of buffer
     nHeight - height of buffer
     fTailFraction - percent, as a fraction, of either end of the histogram
     should be ignored when calculating stats
     pnRedMin, pnRedMax,
     pnGreenMin, pnGreenMax,
     pnBlueMin, pnBlueMax - pointers to storage for the max and min values
     for the three channels
     pfRedMean,
     pfGreenMean,
     pfBlueMean   - pointers to storage for the mean values for the
     three channels
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictGetColorImageStats
(
   void * pColorBuffer,
   ULONG nWidth,
   ULONG nHeight,
   float fTailFraction,
   BYTE * pnRedMin,
   BYTE * pnRedMax,
   float * pfRedMean,
   BYTE * pnGreenMin,
   BYTE * pnGreenMax,
   float * pfGreenMean,
   BYTE * pnBlueMin,
   BYTE * pnBlueMax,
   float * pfBlueMean
   );

   /**********************************************************************
   Balances the three color channels of an RGB image.
   
     arguments:
     pColorBuffer - pointer to RGB image buffer
     nWidth - width of buffer
     nHeight - height of buffer
     nRedMin, nRedMax,
     nGreenMin, nGreenMax,
     nBlueMin, nBlueMax - max and min values for the three channels
     returned by DepictGetColorImageStats
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictBalanceColorImage
(
   void * pColorBuffer,
   ULONG nWidth,
   ULONG nHeight,
   BYTE nRedMin,
   BYTE nRedMax,
   BYTE nGreenMin,
   BYTE nGreenMax,
   BYTE nBlueMin,
   BYTE nBlueMax
);

/**********************************************************************
   Attempts to remove artificts of conversion
   to RGB from mixed.
   
     arguments:
     pColorBuffer - pointer to RGB image buffer
     nWidth - width of buffer
     nHeight - height of buffer
     nFilterSize - Size of neighborhood
     fThreshold - threshold between 0 and 1
               indicating maximum color distance.
               0 is the most stringent, 1 is
               the least
               nNumNeighbours - number of neighbors that must
               be within threshold for pixel
               to pass. Must be between
               1 and (nFilterSize^2 - 1),
               inclusive.
               
                 returns:
                 DEPICT_SUCCESS on success
                 !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictNNFilterColorImage
(
   void * pColorBuffer,
   ULONG nWidth,
   ULONG nHeight,
   ULONG nFilterSize,
   float fThreshold,
   ULONG nNumNeighbours
);

/**********************************************************************
   Converts mixed format image returned by camera into a
   standard 32-bit RGB format image. Conversion uses
   a 3x3 neighborhood around the pixel location to
   interpolate color channel values
   
   The color channels are weighted as follows:
     
   Red   : 1.25
   Green : 1.0
   Blue  : 1.75
       
   arguments:
      pBuffer - pointer to mixed format image buffer
      nMixedWidth - width of mixed format buffer
      nMixedHeight - height of mixed format buffer
      
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertMixedToRGBCorrect
(
   void * pBuffer,
   ULONG nMixedWidth,
   ULONG nMixedHeight,
   BYTE nDiffThresh
);

/**********************************************************************
   Rotates greyscale image 180 degrees
   
   arguments:
      pvBuffer - pointer to image buffer
      ulWidth - width of image
      ulHeight - height of image
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictRotate180
(
   void * pvBuffer,
   ULONG ulWidth,
   ULONG ulHeight
);

/**********************************************************************
   Reduces image resolution by half in each
   direction by performing a 2x2 averaging
   
   arguments:
      pBuffer - pointer to image buffer
      nWidth - width of image
      nHeight - height of image
     
   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall Depict2x2Average
(
   void * pBuffer,
   ULONG nWidth,
   ULONG nHeight
);

/**********************************************************************
   Applies a 3x3 gaussian smoothing kernel to the image.
   
   arguments:
      pvSrcBuffer - pointer to source image buffer
      pvDestBuffer - pointer to source image buffer
      eFormat - image format
      ulWidth - width of source image
      ulHeight - height of source image

   returns:
      DEPICT_SUCCESS on success
      !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall Depict3By3Gauss
(
   const void * pvSrcBuffer,
   void * pvDestBuffer,
   DEPICT_FORMAT eFormat,
   ULONG ulWidth,
   ULONG ulHeight
);

   /**********************************************************************
   Copies contents of one buffer to another
   
     arguments:
     pSrcBuffer - pointer to source
     pDestBuffer - pointer to dest
     nWidth - width of image
     nHeight - height of image
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCopyBuffer
(
   const void * pSrcBuffer,
   void * pDestBuffer,
   ULONG nWidth,
   ULONG nHeight
);

/**********************************************************************
   Copies a rectangular area of one buffer to another
   
     arguments:
     pSrcBuffer - pointer to source
     pDestBuffer - pointer to dest
     nSrcWidth - width of source image
     nSrcHeight - height of source image
     nCopyLeft - left edge of copy area
     nCopyTop - top edge of copy area
     nCopyWidth - width of copy area
     nCopyHeight - height of copy area
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCopyBufferPart
(
   const void * pSrcBuffer,
   ULONG nSrcWidth,
   ULONG nSrcHeight,
   void * pDestBuffer,
   ULONG nDestWidth,
   ULONG nDestHeight,
   ULONG nCopySrcLeft,
   ULONG nCopySrcTop,
   ULONG nCopyDestLeft,
   ULONG nCopyDestTop,
   ULONG nCopyWidth,
   ULONG nCopyHeight
   );

   /**********************************************************************
   Copies an image buffer composed of 8-bit pixel values
   to an image buffer with the same dimensions but
   composed of 32-bit pixel values. Each 8-bit pixel
   becomes the lower 8 bits of the corresponding
   32-bit pixel
   
     arguments:
     pSrcBuffer - pointer to source
     pDestBuffer - pointer to dest
     nWidth - width of image
     nHeight - height of image 
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictCopy8BitTo32Bit
(
   const void * pSrcBuffer,
   void * pDestBuffer,
   ULONG nWidth,
   ULONG nHeight
   );

   /**********************************************************************
   Unpacks an XRGB buffer into three color channel buffers
   
     arguments:
     pSrcBuffer - pointer to source
     pRedDestBuffer - pointer to red dest
     pGreenDestBuffer - pointer to red dest
     pBlueDestBuffer - pointer to red dest
     nWidth - width of image
     nHeight - height of image 
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictUnpackXRGBBuffer
(
   const void * pSrcBuffer,
   void * pRedDestBuffer,
   void * pGreenDestBuffer,
   void * pBlueDestBuffer,
   ULONG dwWidth,
   ULONG dwHeight
   );

   /**********************************************************************
   Unpacks 10 bit per channel RGB buffer into separate 16 bit buffers
   for each channel
   
     arguments:
     pSrcBuffer - pointer to source
     pRedDestBuffer - pointer to red dest
     pGreenDestBuffer - pointer to red dest
     pBlueDestBuffer - pointer to red dest
     nWidth - width of image
     nHeight - height of image 
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictUnpack10BitRGBBuffer
(
   const void * pSrcBuffer,
   void * pRedDestBuffer,
   void * pGreenDestBuffer,
   void * pBlueDestBuffer,
   ULONG dwWidth,
   ULONG dwHeight
   );

   /**********************************************************************
   Converts the three-byte RGB format to four-byte XRGB format
   
     arguments:
     pSrcBuffer - pointer to source
     pDestBuffer - pointer to dest
     nWidth - width of image
     nHeight - height of image 
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertRGBToXRGB
(
   const void * pSrcBuffer,
   void * pDestBuffer,
   ULONG dwWidth,
   ULONG dwHeight
   );

   /**********************************************************************
   Converts the three-byte RGB format to four-byte XRGB format
   and also places the individual channel gray-scale bitmaps
   into separate buffers
   
     arguments:
     pSrcBuffer - pointer to source
     pXRGBDestBuffer - pointer to XRGB dest
     pRedDestBuffer - pointer to dest
     pGreenDestBuffer - pointer to dest
     pBlueDestBuffer - pointer to dest
     nWidth - width of image
     nHeight - height of image 
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertRGBToXRGBAndUnpack
(
   const void * pSrcBuffer,
   void * pXRGBDestBuffer,
   void * pRedDestBuffer,
   void * pGreenDestBuffer,
   void * pBlueDestBuffer,
   ULONG dwWidth,
   ULONG dwHeight
   );

   /**********************************************************************
   Converts the three-byte RGB format to four-byte XRGB format
   and also generates histograms for the three channels
   
     arguments:
     pSrcBuffer - pointer to source
     pXRGBDestBuffer - pointer to XRGB dest
     padwRedHisto - pointer to red histogram
     padwGreenHisto - pointer to green histogram
     padwBlueHisto - pointer to blue histogram
     nWidth - width of image
     nHeight - height of image 
     
       returns:
       DEPICT_SUCCESS on success
       !DEPICT_SUCCESS is an error # 
**********************************************************************/
DEPICT_RESULT _DEPICT_API __stdcall DepictConvertRGBToXRGBAndGenHistos
(
   const void * pSrcBuffer,
   void * pXRGBDestBuffer,
   ULONG * padwRedHisto,
   ULONG * padwGreenHisto,
   ULONG * padwBlueHisto,
   ULONG dwWidth,
   ULONG dwHeight
   );

#ifdef __cplusplus

} /* extern "C" */ 

#endif

#endif /* __DEPICT_H__ */
