#include "Mosaic.h"

#include "jed.h"
#include <stdlib.h>
#include "Msg.h"

Mosaic::Mosaic()
{
  red = new float[1];
  green = new float[1];
  blue = new float[1];
  alpha = new float[1];
  xSize = ySize = 1;
  xmin = xmax = ymin = ymax = 0;
  
}

Mosaic::Mosaic(int Xmin, int Xmax, int Ymin, int Ymax)
{
  xSize = Xmax-Xmin+1;
  ySize = Ymax-Ymin+1;
  
  red = new float[xSize*ySize];
  green = new float[xSize*ySize];
  blue = new float[xSize*ySize];
  alpha = new float[xSize*ySize];
  for (int i=0;i<xSize*ySize;i++) red[i]=0.0;
  for ( i=0;i<xSize*ySize;i++) green[i]=0.0;
  for ( i=0;i<xSize*ySize;i++) blue[i]=0.0;
  for ( i=0;i<xSize*ySize;i++) alpha[i]=0.0;
  
  
  xmin = Xmin;
  xmax = Xmax;
  ymin = Ymin;
  ymax = Ymax;
}

Mosaic::~Mosaic()
{
  delete [] red;
  delete [] green;
  delete [] blue;
  delete [] alpha;
  
}

void Mosaic::mospad(Mosaic &mos)
{
  msg.printRaw("Mosaic:").print(xmin).print(xmax).print(ymin).print(ymax).print("");
  msg.printRaw("Mosaic:").print(mos.xmin).print(mos.xmax).print(mos.ymin).print(mos.ymax).print("");
  
  mospad(mos.xmin, mos.xmax, mos.ymin, mos.ymax);
  msg.printRaw("Done  :").print(xmin).print(xmax).print(ymin).print(ymax).print("");

}

void Mosaic::mospad(int xminNew, int xmaxNew, int yminNew, int ymaxNew)
{
  // If padding does not extend Mosaic then return
  if (xminNew>=xmin && xmaxNew<=xmax && yminNew>=ymin && ymaxNew <=
      ymax)
    return;
  
  // Find extents  of new region
  int minminx = MIN(xminNew, xmin);
  int maxmaxx = MAX(xmaxNew, xmax);
  int minminy = MIN(yminNew, ymin);
  int maxmaxy = MAX(ymaxNew, ymax);
  
  // Create space for new padding
  Mosaic tmpmos(minminx, maxmaxx, minminy, maxmaxy);
  
  // Copy old pixels over
  for (int j=ymin; j<=ymax; j++)
    for (int i=xmin; i<=xmax; i++)
      {
	tmpmos.r(i,j) = r(i,j);
	tmpmos.g(i,j) = g(i,j);
	tmpmos.b(i,j) = b(i,j);
	tmpmos.a(i,j) = a(i,j);
      }
  
  // delete the old storage
  delete [] red;
  delete [] green;
  delete [] blue;
  delete [] alpha;
  
  // move the new values over
  red = tmpmos.red;
  green = tmpmos.green;
  blue = tmpmos.blue;
  alpha = tmpmos.alpha;
  xSize = tmpmos.xSize;
  ySize = tmpmos.ySize;
  xmin = tmpmos.xmin;
  xmax = tmpmos.xmax;
  ymin = tmpmos.ymin;
  ymax = tmpmos.ymax;
  
  // NULL the tmpmos storage, so we dont lose it on destruction
  tmpmos.red = NULL;
  tmpmos.green = NULL;
  tmpmos.blue = NULL;
  tmpmos.alpha = NULL;
  
}

  
void Mosaic::mosadd(Mosaic &mos)
{
  // Extend *this range to include any new area
  mospad(mos);

  // Add in the new information
  for (int j=mos.ymin; j<=mos.ymax; j++)
    for (int i=mos.xmin; i<=mos.xmax; i++)
      {
	r(i,j) += mos.r(i,j);
	g(i,j) += mos.g(i,j);
	b(i,j) += mos.b(i,j);
	a(i,j) += mos.a(i,j);
	
      }
}

void Mosaic::normalizeAlpha()
{
  int fudge=0.0001; // in case of alpha==0.0
  
  for (int j=ymin; j<=ymax; j++)
    for (int i=xmin; i<=xmax; i++)
      {
	r(i,j) /= a(i,j)+fudge;
	g(i,j) /= a(i,j)+fudge;
	b(i,j) /= a(i,j)+fudge;
	a(i,j) = 1.0;
      }
}


