#ifdef WIN32
#include "jd_time.h"
#endif
#include "millitimer.h"
#include "NET_Client.h"
#include <math.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include "CLO.h"

main (int argc, char **argv)
{

  // Handle options
  CLOint errtol(100,"ttolerance","Net sync millisecond max error tolerance");
  CLOint resynctime(10000,"rresync time","Resync time in milliseconds");
  CLOint sleeptime(200,"ssleep time","Sleep time in milliseconds");
  CLOchar255 hostname(NET_HOST_NAME,"hhostname","Network millitimerd host name");
  CLO::parseArgs(argc, argv);
  
  // Get the initial millisecond unsynced time
  long origtime = Get_Milliseconds();
  printf ("\nOriginal milliseconds %d\n\n",origtime);

  // Set Error bounds
  setMillitimerTolerance(errtol);
  
  // Sync with the network
  syncMillitimerWithNetwork(hostname);
  origtime = Get_Milliseconds();
  
  // Loop forever printing the current time
  while (1)
    {
      usleep(1000*sleeptime);
      unsigned long newtime = Get_Milliseconds();
      unsigned long nettime = checkNetworkMillitimer(hostname);

      if ( nettime== -1)
	exit(2);
      
      printf ("Loc: %d   Net: %d   EstNetDiff: %d   NetDiff: %d   MilliAdj: %d\n",
	      newtime, nettime, getMillitimerNetworkError(),
	      abs(newtime-nettime),getMillitimerAdjust());

      // Check if its time to resync
      if ((newtime-origtime) > (int)resynctime)
	{
	  puts ("Re-syncing to network");
	  syncMillitimerWithNetwork(hostname);
	  origtime=Get_Milliseconds();
	}
    }
 

 
}

