#include "NET_Master.h"

#include <fcntl.h>
#include <iostream.h>

#define DEFAULT_PROTO 0
#define QUEUE_LEN 5
//#define SERVER_PORT 7878

/***************************************************************************
                N E T _ M A S T E R : : N E T _ M A S T E R 
***************************************************************************/
NET_Master::NET_Master(int port,bool is_pure)
{
 
	socketStartup();


	// Set clients to NULL
  for (int i=0;i<MAXCLIENTS;i++)
    clientList[i]=NULL;

  mostRecentClientRecv=NULL;

  uniqueIdCounter=-1;

  // Save server port number
  serverPort=port;

  // Start listening for connections
  memset((char *) &serversocket, 0, sizeof(serversocket));
  serversocket.sin_family = AF_INET;
  serversocket.sin_addr.s_addr = htonl(INADDR_ANY);
  serversocket.sin_port = htons(serverPort);
  serverfd = socket(AF_INET, SOCK_STREAM, DEFAULT_PROTO);
  bind(serverfd, (struct sockaddr *)(&serversocket), sizeof(serversocket));
  listen(serverfd, QUEUE_LEN);

  /* set non-blocking */
   setblocking(false);

   pure=is_pure; //default should be false
}

/***************************************************************************
               N E T _ M A S T E R : : ~ N E T _ M A S T E R 
***************************************************************************/
NET_Master::~NET_Master()
{
	for (int i=0;i<MAXCLIENTS;i++)
	  {
	    if (clientList[i] != NULL) {
	      delete clientList[i];
	    }
	  }

	close(serverfd);
	socketCleanup();



}
 
/***************************************************************************
                     * N E T _ M A S T E R : : R E A D 
***************************************************************************/
// We return the client from which a line was read, or NULL
// bufsize is value-result. In=max bytes to receive, Out=bytes read
// bufsize=0 on eof, and -1 on no msges read to read
NET_Client *NET_Master::readNetMsg(void *buf, int *bufsize)
{
NET_Client *nextClient=mostRecentClientRecv;
int strlen = -1;
int nullCount=0;

// Search for either a msg or eof, or end of list
 do {
   nextClient=getNextClient(nextClient);
   if (nextClient==NULL)
     {
       nullCount++;
       // go to beginning of the list
       nextClient=getNextClient(nextClient);
       if (nextClient==NULL)
	 {
	   // error, no next client ?
	   *bufsize = -2;
	   return NULL;
	 }
     }
   nextClient->nonblocking();
   strlen = nextClient->readNetMsg(buf,*bufsize);
 } while (strlen<0 && nextClient!=mostRecentClientRecv && nullCount<2);

 // Update most recent recv checked
 mostRecentClientRecv=nextClient;

 if (strlen==0) //eof, remove this client from the list
   {
     deleteClient(nextClient);

     *bufsize = 0;
     return NULL;
   }
 else if (strlen>0) //msg recv'd
   {
     *bufsize = strlen;
     return nextClient;
   }
 else // no msg avail
   {
     *bufsize = -1;
     return NULL;
   }

}

/***************************************************************************
              N E T _ M A S T E R : : D E L E T E C L I E N T 
***************************************************************************/
void NET_Master::deleteClient(NET_Client *client)  

{
  cout << "NetM: Deleting Client: " << client->clientName << endl;
  
  int last = getClientNum(NULL);
  last--;
  int cur = getClientNum(client);
  delete client;
  if (last >0)
    {
      clientList[cur]=clientList[last];
      clientList[last]=NULL;
    }
  else
    {
      clientList[cur]=NULL;
      if (cur>0)
	puts ("error in client eof");
    }
}

/***************************************************************************
              N E T _ C L I E N T : : G E T C L I E N T N U M 
***************************************************************************/
int NET_Master::getClientNum(NET_Client *cli)

{
 
  int count=0;
  while (clientList[count]!=cli && count<MAXCLIENTS)
    count++;
  
  if (count==MAXCLIENTS)
    return -1;
  else
    return count;
}

/***************************************************************************
            * N E T _ M A S T E R : : G E T N E X T C L I E N T 
***************************************************************************/
NET_Client *NET_Master::getNextClient(NET_Client *cli)
{
  if (cli==NULL) 
    return clientList[0];

  for (int i=0;i<MAXCLIENTS-1;i++)
    if (clientList[i]==cli)
      return clientList[i+1];

  return NULL;
}

/***************************************************************************
        N E T _ M A S T E R : : C H E C K N E W C O N N E C T I O N 
***************************************************************************/
NET_Client *NET_Master::checkNewConnection()
{
  NET_Client *cli=new NET_Client(pure);
 
  //puts("before accept");
  cli->clientfd = accept(serverfd, (struct sockaddr *)&cli->clientsocket,&cli->clientlen);
  //puts ("after accept");

  // Successful connection
#ifdef sgi
  if (cli->clientfd>=0)
#else
  if (cli->clientfd>0)
#endif
    {
      struct hostent *remote_host = gethostbyaddr((char *)(&(cli->clientsocket.sin_addr)),
				       sizeof(struct in_addr), AF_INET);
 #ifndef sgi
  if (remote_host== NULL) {
    int WSAErr = WSAGetLastError();
	printf("gethostbyadd error, error # %d\n",WSAErr);	
  }
#endif
  //printf("XConnection from %s %d\n", remote_host->h_name,
  //	     cli->clientfd );
      strcpy(cli->remotehostname,remote_host->h_name);
       
      // Set non-clocking
      cli->nonblocking();

      // Figure out where to put it in client list
      int i=0;
      while (clientList[i]!=NULL && i<MAXCLIENTS)
	i++;

      if (i<MAXCLIENTS)
	clientList[i]=cli;
      else
	puts ("Error: All connections in use");

      //
      uniqueIdCounter++;
      cli->id=uniqueIdCounter;

      cout << "NETM: Connection:" << i << " " << uniqueIdCounter << endl;
      
      // Get the clientName in the first packet
	  if (!pure) {
		  int gotpacket= -1;
		  while (gotpacket<0)
			  {	
				//puts("about to try to get client name");
				gotpacket=cli->readNetMsg(cli->clientName,254);
			  }		
		  printf ("NETM: client name:%s\n",cli->clientName);
      }
      // Return Client
      return cli;
    }
  // No connection ready
  else
    {
      delete cli;
      return NULL;
    }

  // err=write(clientfd, "Hello, world!\n", 15);

}

/***************************************************************************
         N E T _ M A S T E R : : W A I T N E W C O N N E C T I O N 
***************************************************************************/
NET_Client *NET_Master::waitNewConnection()
{
	// Set blocking
	setblocking(true);
	
	// checkNewConnection
	NET_Client *cli=checkNewConnection();
	
	
	// Unset blocking
	setblocking(false);
	
	// Return client
	return cli;
}

/***************************************************************************
N E T _ M A S T E R : : S E T B L O C K I N G 
***************************************************************************/

void NET_Master::setblocking(int isblocking)
{
	unsigned long arg= !isblocking;
	ioctl(serverfd,FIONBIO,&arg);
}


/***************************************************************************
N E T _ M A S T E R : : G E T B L O C K I N G 
***************************************************************************/

int NET_Master::getblocking()
{
#ifdef sgi
	int flags=fcntl(serverfd, F_GETFL);
	return !(flags & FNONBLK);
#else
	//return WSAIsBlocking();
	return 0;
#endif
}
