//
// EF_Movie.C
//
// $Id: JDMovie.C,v 1.1.1.1 2003/05/21 19:37:52 jedavis Exp $
//


#include <dmedia/moviefile.h>

#include <stdio.h>
#include <dmedia/dm_color.h>
#include <dmedia/dm_image.h>
#include <dmedia/dm_params.h>
#include "JDMovie.h"

/***************************************************************************
                           M O V I E C R E A T E 
***************************************************************************/
void movieCreate(char *filename, MVid &movie, short xsize, short ysize)

{
    DMparams *params;
    MVid track;
  
    if ((dmParamsCreate( &params) !=DM_SUCCESS) ||
	(mvSetMovieDefaults(params, MV_FORMAT_QT)!=DM_SUCCESS) ||
	(mvCreateFile(filename,params,NULL,&movie)!=DM_SUCCESS) ||
	(dmSetImageDefaults(params,xsize,ysize,DM_PACKING_APPLE_32))) {
	fprintf(stderr, "error creating SGI movie\n");
    }
	
    dmParamsSetEnum(params,DM_IMAGE_PACKING, DM_PACKING_APPLE_32);
    dmParamsSetEnum(params,DM_IMAGE_ORIENTATION,  DM_TOP_TO_BOTTOM);
    dmParamsSetString(params,DM_IMAGE_COMPRESSION, DM_IMAGE_JPEG);
    dmParamsSetFloat(params, DM_IMAGE_QUALITY_SPATIAL, .9);
    if (mvAddTrack(movie, DM_IMAGE, params, NULL, &track)!=DM_SUCCESS) {
	fprintf(stderr, "error creating SGI movie\n");
    }
			 
    dmParamsDestroy(params);	
}

/***************************************************************************
                            M O V I E C L O S E 
***************************************************************************/
void movieClose (MVid &movie)

{
  mvClose(movie);
}

/***************************************************************************
                             M O V I E O P E N 
***************************************************************************/
void movieOpen (char *filename, MVid &movie)

{
DMstatus status;

  status=mvOpenFile(filename, O_RDWR, &movie);
  if (status==DM_FAILURE) {
    puts ("error opening movie");
  }
}


/***************************************************************************
                           M O V I E X Y S I Z E 
***************************************************************************/
void movieXYSize(MVid &movie, short &x, short &y)

{
DMstatus status;
MVid track;

  status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
    if (status==DM_FAILURE) {
	puts ("couldnt open movie track");

    }
 
    x=mvGetImageWidth(track);
    y=mvGetImageHeight(track);
}

/***************************************************************************
                        M O V I E N U M F R A M E S 
***************************************************************************/
short movieNumFrames(MVid &movie)

{
   
    DMstatus status;
    MVid track;

    status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
    if (status==DM_FAILURE) {
	puts ("couldnt open movie track");
	mvClose(movie);
	return 0;
    }

    short tracklen = mvGetTrackLength(track);

    return tracklen;
 
}

/***************************************************************************
                         M O V I E G E T F R A M E 
***************************************************************************/
void movieGetFrame(MVid &movie, short framenum, ImageInfo &image)

{
  DMstatus status;
  MVid track;
 
 
  status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
  if (status==DM_FAILURE) {
    puts ("couldnt open movie track");
  
  }
 
  int width=mvGetImageWidth(track);
  int height=mvGetImageHeight(track);
 
  if (width!=image.xsize || height!=image.ysize)
    puts ("ERROR: Movie frame wrong size");
 
  if (image.pixels==NULL)
    puts ("ERROR: NULL pixels");

  JDImage flipimage(width,height);

  status=mvReadFrames(track,framenum,1,width*height*4,flipimage.pixels);
  if (status==DM_FAILURE) {
    puts ("couldnt read frame");
   
  }
 
  movieFlipImageTopBottom(flipimage, image);
  
}

/***************************************************************************
                         M O V I E P U T F R A M E 
***************************************************************************/
void moviePutFrame(MVid &movie, short framenum, ImageInfo &image)

{
  DMstatus status;
  MVid track;

  status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
  if (status==DM_FAILURE) {
    fprintf(stderr, "error creating SGI movie\n");
    return;
  }
 
  int width=mvGetImageWidth(track);
  int height=mvGetImageHeight(track);

  if (width!=image.xsize || height!=image.ysize)
    puts ("ERROR: Movie frame wrong size");
 
  if (image.pixels==NULL)
    puts ("ERROR: NULL pixels");
  
  // convert to xbgr
   JDImage xbgrimage(image.xsize,image.ysize);
  // movieRGBAtoXBGR(image, xbgrimage);
   movieFlipImageTopBottom(image, xbgrimage);
  //ImageInfo xbgrimage=image;

  // Write frame
  status=mvDeleteFrames(track,framenum,1);
  if (status==DM_FAILURE) {
    fprintf(stderr, "error creating SGI movie\n");
    return;
  }

  status=mvInsertFrames(track,framenum,1,width*height*4,xbgrimage.pixels);
  if (status==DM_FAILURE) {
    fprintf(stderr, "error creating SGI movie\n");
    return;
  }

 
 
}




/***************************************************************************
                       M O V I E X B G R T O R G B A 
***************************************************************************/
void movieXBGRtoRGBA(ImageInfo &image, ImageInfo &outimage)

{
    char *outbuf=NULL;
    DMcolorconverter convert;
    DMparams *srcP, *destP;
  
    short xsize=image.xsize;
    short ysize=image.ysize;
   
    outbuf = (char *)outimage.pixels;
   
    dmColorCreate(&convert);
     
    dmParamsCreate(&srcP);
    dmParamsSetInt(srcP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(srcP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(srcP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_XBGR );
    dmParamsSetEnum(srcP, DM_IMAGE_ORIENTATION,
		    DM_IMAGE_TOP_TO_BOTTOM);
    dmColorSetSrcParams(convert,srcP);

    dmParamsCreate(&destP);
    dmParamsSetInt(destP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(destP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(destP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_RGBA);
    dmParamsSetEnum(destP, DM_IMAGE_ORIENTATION, 
		    DM_IMAGE_BOTTOM_TO_TOP);
    dmColorSetDstParams(convert,destP );
      
    dmColorConvert(convert, image.pixels, outbuf);

    dmParamsDestroy(srcP);
    dmParamsDestroy(destP);
  
    dmColorDestroy(convert);

  
}

/***************************************************************************
                       M O V I E R G B A T O X B G R 
***************************************************************************/
void movieRGBAtoXBGR(ImageInfo &image, ImageInfo &outimage)

{
    char *outbuf=NULL;
    DMcolorconverter convert;
    DMparams *srcP, *destP;
  
    short xsize=image.xsize;
    short ysize=image.ysize;
   
    outbuf = (char *)outimage.pixels;
   
    dmColorCreate(&convert);
     
    dmParamsCreate(&srcP);
    dmParamsSetInt(srcP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(srcP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(srcP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_RGBA );
    dmParamsSetEnum(srcP, DM_IMAGE_ORIENTATION,
		    DM_IMAGE_TOP_TO_BOTTOM);
    dmColorSetSrcParams(convert,srcP);

    dmParamsCreate(&destP);
    dmParamsSetInt(destP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(destP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(destP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_XBGR);
    dmParamsSetEnum(destP, DM_IMAGE_ORIENTATION, 
		    DM_IMAGE_BOTTOM_TO_TOP);
    dmColorSetDstParams(convert,destP );
      
    dmColorConvert(convert, image.pixels, outbuf);

    dmParamsDestroy(srcP);
    dmParamsDestroy(destP);

    dmColorDestroy(convert);

  
}

/***************************************************************************
               M O V I E F L I P I M A G E T O P B O T T O M 
***************************************************************************/
void movieFlipImageTopBottom(ImageInfo &image, ImageInfo &outimage)

{
    char *outbuf=NULL;
    DMcolorconverter convert;
    DMparams *srcP, *destP;
  
    short xsize=image.xsize;
    short ysize=image.ysize;
   
    outbuf = (char *)outimage.pixels;
   
    dmColorCreate(&convert);
     
    dmParamsCreate(&srcP);
    dmParamsSetInt(srcP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(srcP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(srcP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_RGBA );
    dmParamsSetEnum(srcP, DM_IMAGE_ORIENTATION,
		    DM_IMAGE_TOP_TO_BOTTOM);
    dmColorSetSrcParams(convert,srcP);

    dmParamsCreate(&destP);
    dmParamsSetInt(destP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(destP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(destP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_RGBA);
    dmParamsSetEnum(destP, DM_IMAGE_ORIENTATION, 
		    DM_IMAGE_BOTTOM_TO_TOP);
    dmColorSetDstParams(convert,destP );
      
    dmColorConvert(convert, image.pixels, outbuf);

    dmParamsDestroy(srcP);
    dmParamsDestroy(destP);

    dmColorDestroy(convert);
}
