#include "error.h"
#include "glutwm.h"
#include "moviewindow.h"

MovieWindow::
MovieWindow(void)
{
    full = false;
    old_width = old_height = 0;
}

void 
MovieWindow::
Display(void)
{
    OwnerImageWindowUchar::Display();
}

int 
MovieWindow::
LoadFrame(int frame)
{
    if (movie.Read(frame, GetBuffer())) {
        debugf(("Frame %d", frame));
        current = frame;
        // need immediate update
        Display(); GlutWMSwapBuffers();
        return 1;
    } else {
        warnf(("Error loading frame %d", frame));
        return 0;
    }
}

int 
MovieWindow::
GetLength(void)
{
    return movie.GetLength();
}

int 
MovieWindow::
CycleFrame(void)
{
    current++;
    if (current >= movie.GetLength()) current = 0;
    return LoadFrame(current);
}

int 
MovieWindow::
Load(const char *name)
{
    if (!movie.Load(name) || !LoadFrame(0)) return 0;
    Fit();
    return 1;
}

int 
MovieWindow::
NextFrame(void)
{
    if (current+1 < movie.GetLength()) return LoadFrame(current+1);
    return 0;
}

int 
MovieWindow::
PreviousFrame(void)
{
    if (current-1 >= 0) return LoadFrame(current-1);
    return 0;
}


void 
MovieWindow::
Keyboard(unsigned char c, int x, int y)
{
    switch (c) {
        case ',': 
            PreviousFrame(); 
            break;
        case '.': 
            NextFrame(); 
            break;
        case 'm':
            if (full) {
                GlutWMReshapeWindow(GetId(), old_width, old_height);
                full = false;
            } else {
                old_width = GetWidth();
                old_height = GetHeight();
                GlutWMFullScreen(GetId());
                full = true;
            }
            break;
        default:
            OwnerImageWindowUchar::Keyboard(c, x, y); 
            break;
    }
}
