#include "plysoup.h"
#include "rply.h"
#include "error.h"

PlySoup::
PlySoup(void)
{
    ;
}

PlySoup::
~PlySoup(void)
{
    ;
}

int
PlySoup::
Store(const char *name)
{
    p_ply ply = ply_create(name, PLY_LITTLE);
    if (!ply) {
        errorf(("Unable to open file for writing"));
        goto error;
    }
    if (!ply_add_element(ply, "vertex", vertex.GetSize())) goto error;
    if (!ply_add_property(ply, "x", PLY_FLOAT, 0, 0)) goto error;
    if (!ply_add_property(ply, "y", PLY_FLOAT, 0, 0)) goto error;
    if (!ply_add_property(ply, "z", PLY_FLOAT, 0, 0)) goto error;
    if (!ply_add_element(ply, "face", triangle.GetSize())) goto error;
    if (!ply_add_property(ply, "vertex_indices", 
            PLY_LIST, PLY_UCHAR, PLY_INT)) goto error;
    if (!ply_write_header(ply)) {
        errorf(("Error writing header"));
        goto error;
    }
    for (int i = 0; i < vertex.GetSize(); i++) {
        if (!ply_write(ply, vertex[i][X])) goto error;
        if (!ply_write(ply, vertex[i][Y])) goto error;
        if (!ply_write(ply, vertex[i][Z])) goto error;
    }
    for (int i = 0; i < triangle.GetSize(); i++) {
        if (!ply_write(ply, 3)) goto error;
        if (!ply_write(ply, triangle[i].index[0])) goto error;
        if (!ply_write(ply, triangle[i].index[1])) goto error;
        if (!ply_write(ply, triangle[i].index[2])) goto error;
    }
    if (!ply_close(ply)) goto error;
    debugf(("Done writing %d vertices, %d triangles", vertex.GetSize(), 
                triangle.GetSize()));
    return 1;
error:
    errorf(("Something went wrong"));
    if (ply) ply_close(ply);
    return 0;
}

static int vertex_cb(p_ply_argument argument)
{
    Soup *soup = (Soup *) argument->pdata;
    if (argument->idata == 0 && !soup->vertex.PushBack()) return 0;
    soup->vertex[argument->element_index][argument->idata] = argument->value;
    return 1;
}

static int face_cb(p_ply_argument argument)
{
    Soup *soup = (Soup *) argument->pdata;
    if (argument->nentries != 3) return 0;
    if (argument->entry_index == 0 && !soup->triangle.PushBack()) return 0;
    soup->triangle[argument->element_index].index[argument->entry_index] = 
        (int) argument->value;
    return 1;
}

int 
PlySoup::
Load(const char *name)
{
    p_ply ply = ply_open(name);
    if (!ply) {
       errorf(("Error openning file"));
       return 0;
    }
    if (!ply_read_header(ply)) {
       errorf(("Invalid header"));
       return 0;
    }
    if (!ply_set_read_cb(ply, "vertex", "x", vertex_cb, this, X)) {
        errorf(("Expected 'x' property of 'vertex' element"));
        return 0;
    }
    if (!ply_set_read_cb(ply, "vertex", "y", vertex_cb, this, Y)) {
        errorf(("Expected 'y' property of 'vertex' element"));
        return 0;
    }
    if (!ply_set_read_cb(ply, "vertex", "z", vertex_cb, this, Z)) {
        errorf(("Expected 'z' property of 'vertex' element"));
        return 0;
    }
    if (!ply_set_read_cb(ply, "face", "vertex_indices", face_cb, this, 0)) {
        errorf(("Expected 'vertex_indices' property of 'face' element"));
        return 0;
    }
    vertex.Empty();
    triangle.Empty();
    if (!ply_read(ply)) {
       errorf(("Error reading file"));
       return 0;
    }
    ply_close(ply);
    debugf(("Done loading %d vertices, %d triangles", vertex.GetSize(),
                triangle.GetSize()));
    return 1;
}
