#include "error.h"
#include "pngimage.h"
#include "pngtarmovie.h"

PngTarMovie::
PngTarMovie(void)
{
    append_entry = 0;
}

PngTarMovie::
~PngTarMovie()
{
    ;
}

int 
PngTarMovie::
Load(const char *name)
{
    if (!archive.Load(name)) {
        errorf(("Unable to load archive '%s'", name));
        return 0;
    }
    return 1;
}

int 
PngTarMovie::
Create(const char *name)
{
    if (!archive.Create(name)) {
        errorf(("Unable to create archive '%s'", name));
        return 0;
    }
    return 1;
}

int 
PngTarMovie::
Append(const BufferUchar &buffer)
{
    PngImage image;
    if (!image.Load(buffer)) {
        errorf(("Unable to convert buffer to PNG format"));
        return 0;
    }
    char name[20];
    sprintf(name, "%05d.png", append_entry);
    FILE *fp = archive.AppendBegin(name);
    if (!fp) {
        errorf(("Unable to start appending"));
        return 0;
    }
    int ret = 0;
    if (!image.Store(fp)) {
        errorf(("Unable to append image to archive"));
    } else ret = 1;
    if (!archive.AppendEnd()) {
        errorf(("Unable to finalize appending"));
        return  0;
    }
    append_entry++;
    return ret;
}

int
PngTarMovie::
Close(void)
{
    return archive.Close();
}

int 
PngTarMovie::
Read(int frame, BufferUchar &buffer)
{
    if (frame < 0 || frame >= (int) archive.GetNEntries()) {
        errorf(("Frame %d is out of range", frame));
        return 0;
    }
    BufferUchar blob;
    if (!blob.Resize(archive.GetSize(frame), 1, 1) 
            || !archive.Read(frame, blob.GetBuffer())) {
        errorf(("Unable to load frame %d from archive", frame));
        return 0;
    }
    PngImage image;
    if (!image.Load(blob.GetBuffer(), blob.GetSize())) {
        errorf(("Frame %d is not in a recognized image format", frame));
        return 0;
    }
    if (!image.Store(buffer)) {
        errorf(("Unable to store frame %d in buffer", frame));
        return 0;
    }
    return 1;
}

int 
PngTarMovie::
GetLength(void)
{
    return archive.GetNEntries();
}
