#ifndef SCALAR_H
#define SCALAR_H

#include <cmath>
#include <climits>

#define SCALAR_MIN (-1e6)
#define SCALAR_MAX (1e6)
#define SCALAR_EPSILON (1/SCALAR_MAX)
#define SCALAR_PI (3.141592653589)
#define SCALAR_TODEGREE (180.0/SCALAR_PI)
#define SCALAR_TORADIAN (SCALAR_PI/180.0)

#ifndef MAX
#define MAX(a, b) ((a > b) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a, b) ((a > b) ? (a) : (b))
#endif

template <class T> 
static inline void Swap(T &a, T &b)
{
    T temp = a;
    a = b;
    b = temp;
}

inline bool Zero(const double &a)
{
    return fabs(a) <= SCALAR_EPSILON;
}

inline bool Zero(const float &a)
{
    return fabs(a) <= SCALAR_EPSILON;
}

template <class T>
inline bool Equal(const T& a, const T& b)
{
    return Zero(a-b);
}

template <class T>
inline bool Positive(const T& a)
{
    return a > (T) SCALAR_EPSILON;
}

template <class T>
inline bool Negative(const T& a)
{
    return a < (T) (-SCALAR_EPSILON);
}

typedef unsigned char Uchar;
typedef unsigned short Ushort;
typedef unsigned long Ulong;

inline double Limit(const Uchar &c) { (void) c; return UCHAR_MAX; }
inline double Limit(const Ushort &c) { (void) c; return USHRT_MAX; }
inline double Limit(const Ulong &c) { (void) c; return ULONG_MAX; }
inline double Limit(const float &c) { (void) c; return 1.0; }
inline double Limit(const double &c) { (void) c; return 1.0; }

template <class T, class U>
inline void Translate(T &dest, const U& source)
{ dest = (T) (source * Limit(dest)/Limit(source)); }


#endif // SCALAR_H
