#ifndef TAR_H
#define TAR_H

#include <cstdio>
#include "array.h"

#define TAR_BLOCKSIZE     512
#define TAR_NAMESIZE      100
#define TAR_OLD_NORMAL    '\0'
#define TAR_NORMAL        '0'

#define TAR_UREAD         00400ul
#define TAR_UWRITE        00200ul
#define TAR_UEXEC         00100ul
#define TAR_GREAD         00040ul
#define TAR_GWRITE        00020ul
#define TAR_GEXEC         00010ul
#define TAR_OREAD         00004ul
#define TAR_OWRITE        00002ul
#define TAR_OEXEC         00001ul

typedef struct t_header_ {
    char name[TAR_NAMESIZE];
    char mode[8];
    char uid[8];
    char gid[8];
    char size[12];
    char mtime[12];
    char checksum[8];
    char linkflag;
    char linkname[TAR_NAMESIZE];
} t_header;
typedef t_header *p_header;

typedef union u_block_ {
    unsigned char data[TAR_BLOCKSIZE];
    t_header header;
} u_block;
typedef u_block *p_block;

typedef struct t_entry_ {
    size_t start, size;
} t_entry;

enum {TAR_READ, TAR_WRITE, TAR_CLOSED};

class Tar {
    public:
        Tar(void); 
        ~Tar();
        int Load(const char *name);
        int Read(int entry, unsigned char *data);

        int Create(const char *name);
        FILE *AppendBegin(const char *name);
        int AppendEnd(void);

        int Close(void);
        size_t GetSize(int entry) const;
        size_t GetNEntries(void) const;

    private:
        Array<t_entry> index;
        FILE *fp;
        int mode;
        size_t append_pos;
        char append_name[TAR_NAMESIZE];
};

#endif // TAR_H
