#ifndef TRANS_H
#define TRANS_H
//=======================================================================
// Translation class implementation.
// 19/7/2002, Diego Nehab
//=======================================================================

//=======================================================================
// Trans Class
//=======================================================================
class Trans {
public:
    //---------------------------------------------------------------------
    // Constructor
    //---------------------------------------------------------------------
    Trans(double w = 1, double h = 1, double x0 = 0, double y0 = 0);

    //---------------------------------------------------------------------
    // Mouse down event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Down(double x, double y);

    //---------------------------------------------------------------------
    // Mouse motion event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Motion(double x, double y);

    //---------------------------------------------------------------------
    // Set translation value
    // Input
    //     t: translation amount
    //---------------------------------------------------------------------
    void Set(const Vector3 &t = Vector3(0,0,0));

    //---------------------------------------------------------------------
    // Reset viewport and projection parameters
    //---------------------------------------------------------------------
    void Reset(double w, double h, double x0, double y0);

    //---------------------------------------------------------------------
    // Mouse down event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Up(double x, double y);

    //---------------------------------------------------------------------
    // Get cumulative translation
    //---------------------------------------------------------------------
    Vector3 Translation(void) const;
    void Translation(double *x, double *y, double *z) const;

private:
    Vector3 getv(double x, double y);
    // are we dragging?
    int drag;
    // cumulative translation, mouse down position and current position
    Vector3 t, p, m; 
    // viewport
    double w, h, x0, y0;
};

//=======================================================================
// Public Methods
//=======================================================================
inline 
Trans::Trans(double w, double h, double x0, double y0)
{
    Reset(x0, y0, w, h);
    drag = false;
}

inline void 
Trans::Reset(double w_, double h_, double x0_, double y0_)
{
    x0 = x0_;
    y0 = y0_;
    w = w_;
    h = h_;
}

inline void 
Trans::Down(double x, double y)
{
    p = m = getv(x, y);
    drag = true;
}

inline void
Trans::Up(double x, double y)
{
    t += getv(x, y) - p;
    drag = false;
    m = p = Vector3(0,0,0); 
}

inline void
Trans::Motion(double x, double y)
{
    if (drag) m = getv(x, y);
}

inline void
Trans::Set(const Vector3 &ut)
{
    t = ut;
}

inline Vector3
Trans::Translation(void) const
{
    return t + m - p;
}

inline void 
Trans::Translation(double *x, double *y, double *z) const
{
    Vector3 r = t + m - p;
    *x = r[X];
    *y = r[Y];
    *z = r[Z];
}

inline Vector3
Trans::getv(double x, double y)
{
    x = (x - x0)/w - 0.5;
    y = (y - y0)/h - 0.5;
    return Vector3(x, y, 1);
}

#endif // TRANS_H
