
#include "vidInterfaceVisionSDK.h"

#include "ImageWin.h"
#include <iostream.h>
#include "glutTools.h"
#include "pngtarmovie.h"
#include "jed.h"
#include "FileNameTools.h"

//using namespace std;

bool gUsingTarMovie=false;

class MovieWin : ImageWin {
public:
	int curFrameNum;
	MVid movie;
	PngTarMovie omovie;
	JDImage curFrame;
	bool frameNeedsLoading;
	bool currentlyPlaying;
	
	MovieWin() {
		curFrameNum=0; 
		setIdleInterval(25);
		frameNeedsLoading=false;
		currentlyPlaying=false;

		activateMenu(GLUT_RIGHT_BUTTON);
		glutAddMenuEntry("Next frame (period)",'.');
		glutAddMenuEntry("Prev frame (comma)",',');
		glutAddMenuEntry("Play       (space)",' ');
		glutAddMenuEntry("Quit       (q)",'q');
		
		
	}

	// This idle() routine gets run every 200ms
	// displaying the image is handled by the routine in ImageWin
	/*void idle() {

		// Increment the current frame number
		curFrameNum = (curFrameNum+1)%movieNumFrames(movie);

		// Get the correct frame from the movie file
		movieGetFrame(movie, curFrameNum, curFrame);

		// Make this the current image for ImageWin->display();
		attachImage(curFrame);

		// Print a diagnostic
		printf ("Showing frame: %d\n",curFrameNum);
	}*/

	int movieNumFrames(MVid &movie) {
		if (gUsingTarMovie) {
			return omovie.GetLength();
		} else {
			return ::movieNumFrames(movie);
		}
	}

	void preloadCurFrame()
	{
		frameNeedsLoading=true;
		glutPostRedisplay();
	}

	void loadCurFrame()
	{
		// Get the correct frame from the movie file
		if (gUsingTarMovie) {
			BufferUchar ibuf;

			if (omovie.Read(curFrameNum,ibuf)) {
				ibuf.Flip();
				curFrame.resize(ibuf.GetWidth(), ibuf.GetHeight());
				ibuf.Store(((unsigned char *)curFrame.pixels)+0,4,0,0);
				ibuf.Store(((unsigned char *)curFrame.pixels)+1,4,0,1);
				ibuf.Store(((unsigned char *)curFrame.pixels)+2,4,0,2);
			} else {
				COUT << "Error reading frame " << curFrameNum << endl;
				return;
			}
			
		} else {
			movieGetFrame(movie, curFrameNum, curFrame);
		}
		
		// Make this the current image for ImageWin->display();
		attachImage(curFrame);
		
		// Print a diagnostic
		// printf ("Showing frame: %d\n",curFrameNum);
	}

	void mouse(int button, int state, int x, int y)
	{
		motion(x,y);
	}

	void motion(int x, int y)
	{
		double xf=x;
		double wf=width;
		double percent=xf/wf;
		//cout << percent << endl;

		double newFrameNum=movieNumFrames(movie)*percent;
		newFrameNum=max(0,newFrameNum);
		newFrameNum=min(movieNumFrames(movie)-1,newFrameNum);
		
		if (newFrameNum!=curFrameNum) {
			curFrameNum=newFrameNum;
			preloadCurFrame();
		}
	}

	void keyboard(unsigned char key, int x, int y)
	{
		switch (key) {
						
			// Prev frame
		case ',':
			curFrameNum = (curFrameNum-1+movieNumFrames(movie))%movieNumFrames(movie);
			preloadCurFrame();
			break;
			
			// Next frame
		case '.':
			curFrameNum = (curFrameNum+1)%movieNumFrames(movie);
			preloadCurFrame();
			break;

			// Play
		case ' ':
			currentlyPlaying= !currentlyPlaying;
			preloadCurFrame();
			break;
			
			// Quit
		case 'q':
			exit(0);
			break;
		}
	} 
	
	void menu(int whichMenu, int value)
	{
		keyboard(value,0,0);
	}
	
	void display() {

		// Check if we are up to date
		if (frameNeedsLoading) {
			loadCurFrame();
			frameNeedsLoading=false;
		}

		// Show the image
		ImageWin::display();

		// Text overlay
		char buf[120];
		sprintf(buf,"Frame Number %d",curFrameNum);
		glutSetStringPos(20,20);
		glutBitmapString(buf);
		//glFlush();
		glutSwapBuffers();
		
	} 
	
	void idle() {
		
		// If we are currently playing advance to the next frame
		if (currentlyPlaying) {
			curFrameNum = (curFrameNum+1)%movieNumFrames(movie);
			preloadCurFrame();
		}
		
	}
	
};


int main(int argc, const char *argv[])
{

	// Check for filename argument
    if ( argc < 2 ) {
		fprintf(stderr,"    \n");
		fprintf(stderr,"Usage: moviecheck input.avi\n");
		fprintf(stderr,"       moviecheck input.png.tar\n");
		fprintf(stderr,"    \n");
		fprintf(stderr,"       Left mouse to scrub, right mouse for menu\n");
		fprintf(stderr,"    \n");
		return -1;
	}

	// Trying to hide the console window, but also kills cygwin window
	//SetConsoleTitle ("<Your fancy text>");
    //HWND hwnd = FindWindow (NULL, "<Your fancy text>");
    //ShowWindow (hwnd, SW_HIDE);

	glutInitDisplayMode(GLUT_DOUBLE);

	// Create a window that knows how to play movies
	MovieWin mwin;

	//std::string title=std::string("MovieCheck (right button for menu)");
	glutSetWindowTitle(argv[1]);

	if (!strcmp(argv[1]+strlen(argv[1])-3,"tar")) {
		gUsingTarMovie=true;
	}

	// Open the movie file in question
	if (gUsingTarMovie) {
		// Open tar movie
		if (!mwin.omovie.Load(argv[1])) {
			COUT << "Error" << endl;
			return -2;
		}

	} else {
		// Open avi movie
		movieOpen(argv[1],mwin.movie);
	}
	
	// Load the first frame
	mwin.loadCurFrame();

	// Enter the glut main loop, which falls through to idle() above
	glutMainLoop();

    return 0;
}


